package clustering;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/**
 * Add a number in front of sequence id to identify each sequence.
 * The number and sequence id are separated by "$". 
 */

/**
 * @author zhiliangc Compute distance matrix based on Levinshtein distance using
 *         gap penalty and difference between V gene.
 *         The output is for R hierarchical clustering.
 */
public class DistanceMatrix {
	private List<String> names;

	private List<String> seqs;

	private List<String> vgenes;

	private List<String> jgenes;

	private double[][] distanceMatrix;

	// private String inputFilename;
	//
	// private String outputFilename;

	// private int Vvalue;

	public int NEW_LD(String s, String t) {
		if (s == null || t == null) {
		    throw new IllegalArgumentException("Strings must not be null");
		  }	
		  int n = s.length(); // length of s
		  int m = t.length(); // length of t
				
		  if (n == 0) {
		    return m;
		  } else if (m == 0) {
		    return n;
		  }

		  int p[] = new int[n+1]; //'previous' cost array, horizontally
		  int d[] = new int[n+1]; // cost array, horizontally
		  int _d[]; //placeholder to assist in swapping p and d

		  // indexes into strings s and t
		  int i; // iterates through s
		  int j; // iterates through t

		  String t_j; // jth character of t
		  String s_i;
		  
		  int cost; // cost

		  for (i = 0; i<=n; i++) {
		     p[i] = i;
		  }
				
		  for (j = 1; j<=m; j++) {
		     t_j = Character.toString(t.charAt(j-1));
		     d[0] = j;
				
		     for (i=1; i<=n; i++) {
		    	 s_i = Character.toString(s.charAt(i-1));
		    	 if (t_j.equalsIgnoreCase(s_i)) {
						cost = 0;
					} else {
						cost = 1;
					}
		        //cost = s.charAt(i-1)==t_j ? 0 : 1;
		        // minimum of cell to the left+1, to the top+1, diagonally left and up +cost				
		        d[i] = Math.min(Math.min(d[i-1]+3, p[i]+3),  p[i-1]+cost);  
		     }
		     // copy current distance counts to 'previous row' distance counts
		     _d = p;
		     p = d;
		     d = _d;
		     
		  } 
				
		  // our last action in the above loop was to switch d and p, so p now 
		  // actually has the most recent cost counts
		  //System.out.println(d);
		  //System.out.println(p);
		  return p[n];


	}
	  public int LD (String s, String t) {
		  int d[][]; // matrix
		  int n; // length of s
		  int m; // length of t
		  int i; // iterates through s
		  int j; // iterates through t
		  //char s_i; // ith character of s
		  //char t_j; // jth character of t
		  int cost; // cost

		    // Step 1

		    n = s.length ();
		    m = t.length ();
		    if (n == 0) {
		      return m;
		    }
		    if (m == 0) {
		      return n;
		    }
		    d = new int[n+1][m+1];

		    // Step 2

		    for (i = 0; i <= n; i++) {
		      d[i][0] = i;
		    }

		    for (j = 0; j <= m; j++) {
		      d[0][j] = j;
		    }

		    // Step 3

		    for (i = 1; i <= n; i++) {

		      String s_i = Character.toString(s.charAt (i - 1));

		      // Step 4

		      for (j = 1; j <= m; j++) {

		      String  t_j = Character.toString(t.charAt (j - 1));

		        // Step 5

		        if (t_j.equalsIgnoreCase(s_i)) {
		          cost = 0;
		        }
		        else {
		          cost = 1;
		        }

		        // Step 6

		        d[i][j] = Math.min(Math.min (d[i-1][j]+3, d[i][j-1]+3), d[i-1][j-1] + cost);

		      }

		    }

		    // Step 7

		    return d[n][m];

		  }



	public int[] NEW_NED(String s, String t) {
		//int d[][]; // matrix
		int n; // length of s
		int m; // length of t
		int i; // iterates through s
		int j; // iterates through t
		int cost; // cost
		 String t_j; // jth character of t
		  String s_i;
		// Step 1
		n = s.length();
		m = t.length();
		int[] results = new int[2];
		int[][] pointerMatrix = new int[n + 1][m + 1];
		int p[] = new int[n+1]; //'previous' cost array, horizontally
		  int d[] = new int[n+1]; // cost array, horizontally
		  int _d[]; //placeholder to assist in swapping p and d
		// Step 2 Initialize
		for (i = 0; i <= n; i++) {
			pointerMatrix[i][0] = 2;
			p[i] = i;
		}
		for (j = 0; j <= m; j++) {
			pointerMatrix[0][j] = 3;
		}
		pointerMatrix[0][0] = 0;
		// Step 3
		for (i = 0; i<=n; i++) {
		     p[i] = i;
		  }
		// Step 4~6		
		for (j = 1; j<=m; j++) {
			 t_j = Character.toString(t.charAt(j-1));
		     d[0] = j;
				
		     for (i=1; i<=n; i++) {
		    	 s_i = Character.toString(s.charAt(i-1));
		    	 if (t_j.equalsIgnoreCase(s_i)) {
						cost = 0;
					} else {
						cost = 1;
					}
		        // minimum of cell to the left+1, to the top+1, diagonally left and up +cost				
				int q, w, r;
				q = d[i-1]+3;
				w = p[i]+3;
				r = p[i-1]+cost;
				d[i] = Math.min(Math.min(q, w), r);
				//d[i] = Math.min(Math.min(d[i-1]+3, p[i]+3),  p[i-1]+cost);
				if (d[i] == r) { // diag
					pointerMatrix[i][j] = 1;
				} else if (d[i] == w) { // top
					pointerMatrix[i][j] = 2;
				} else { // left
					pointerMatrix[i][j] = 3;
				}
		     }
		     // copy current distance counts to 'previous row' distance counts
		     _d = p;
		     p = d;
		     d = _d;
		     
		  } 
		// Step 7 Trace back
		int k = pointerMatrix[n][m];
		int steps = m;
		i = n ;
		j = m ;
		while (k != 0) {
			if (k == 1) {
				j = j - 1;
				i = i - 1;
			} else if (k == 2) {
				i = i - 1;
				steps = steps + 1;
			} else {
				j = j - 1;
				
			}

			k = pointerMatrix[i][j];
		}
		// Step 8

		results[0] = p[n];
		results[1] = steps;

		return results;

	}
	public int[] NED(String s, String t) {
		int d[][]; // matrix
		int n; // length of s
		int m; // length of t
		int i; // iterates through s
		int j; // iterates through t
		int cost; // cost
		// Step 1
		n = s.length();
		m = t.length();
		int[] results = new int[2];
		int[][] pointerMatrix = new int[m + 1][n + 1];
		d = new int[m + 1][n + 1];
		// Step 2 Initialize
		for (i = 0; i <= m; i++) {
			d[i][0] = i;
			pointerMatrix[i][0] = 2;
		}
		for (j = 0; j <= n; j++) {
			d[0][j] = j;
			pointerMatrix[0][j] = 3;
		}
		pointerMatrix[0][0] = 0;
		// Step 3
		for (i = 1; i <= m; i++) {
			// Step 4
			for (j = 1; j <= n; j++) {
				String t_i = Character.toString(t.charAt(i - 1));
				String s_j = Character.toString(s.charAt(j - 1));
				// Step 5
				if (t_i.equalsIgnoreCase(s_j)) {
					cost = 0;
				} else {
					cost = 1;
				}
				// Step 6 update distance value and pointerMatrix
				int p, q, r;
				p = d[i - 1][j] + 3;
				q = d[i][j - 1] + 3;
				r = d[i - 1][j - 1] + cost;
				d[i][j] = Math.min(Math.min(p, q), r);
				if (d[i][j] == r) { // diag
					pointerMatrix[i][j] = 1;
				} else if (d[i][j] == p) { // top
					pointerMatrix[i][j] = 2;
				} else { // left
					pointerMatrix[i][j] = 3;
				}

			}
		}
		// Step 7 Trace back
		int k = pointerMatrix[m][n];
		int steps = n;
		i = m - 1;
		j = n - 1;
		while (k != 0) {
			if (k == 1) {
				j = j - 1;
				i = i - 1;
			} else if (k == 2) {
				i = i - 1;
				steps = steps + 1;
			} else {
				j = j - 1;
			}

			k = pointerMatrix[i][j];
		}

		// Step 8

		results[0] = d[m][n];
		results[1] = steps;

		return results;

	}
	
	public int VMatrix(String x, String y) {
		/*
		 * Example of V gene:IGHV3-72*01_(L1) or IGHV3-30-3*01_(L1)
		 * IGHV3 - V gene family 72 - V gene
		 * name 01 - V gene allele (L1) - confidence level
		 */
		int vvalue = 0;
		String[] xVinput = x.split("[-\\*_]"); // split V gene into gene name
												// and alleles

		/*
		 * String xVfam = xVinput[0]; // V gene family String[] xVgene =
		 * xVinput[1].split("\\*"); String xVname = xVgene[0]; // V gene name
		 * System.out.println(xVname); String[] xVrest = xVgene[1].split("_");
		 * String xVallele = xVrest[0];
		 */
		int xsize = xVinput.length;
		String xVfam = xVinput[0];
		String xVname = xVinput[xsize - 2];
		String xVallele = xVinput[xsize - 3];
		// V gene allele
		// Handle with another V gene input
/*		String[] yVinput = y.split("-");
		String yVfam = yVinput[0];
		String[] yVgene = yVinput[1].split("\\*");
		String yVname = yVgene[0];
		String[] yVrest = yVgene[1].split("_");
		String yVallele = yVrest[0];*/
		// if (xV)
		String[] yVinput = y.split("[-\\*_]"); 
		int ysize = yVinput.length;
		String yVfam = yVinput[0];
		String yVname = yVinput[ysize - 2];
		String yVallele = yVinput[ysize - 3];
		
		if (xVfam.equals(yVfam)) {
			if (xVname.equals(yVname)) {
				if (xVallele.equals(yVallele)) {
					vvalue = 0;
					if (!xVallele.equals(yVallele)) {
						vvalue = 1;
					}

				} else {
					vvalue = 3;
				}
			}
		} else {
			vvalue = 5;
		}
		return vvalue;
	}

	public int JMatrix(String x, String y) {
		/*
		 * Example of J gene:,X86355_IGHJ5*02 IGHV5 - J gene name 02 - V gene
		 * allele
		 */
		int jvalue = 0;
		String[] xJgene = x.split("[_\\*]"); // split V gene into gene
		// name and
		// alleles
		String xJname = xJgene[1];
		String xJallele = xJgene[2];
		String[] yJgene = y.split("[_\\*]");
		String yJname = yJgene[1];
		String yJallele = yJgene[2];
		if (xJname == yJname) {
			if (xJallele == yJallele) {
				jvalue = 0;
			} else {
				jvalue = 1;
			}
		} else {
			jvalue = 3;
		}
		return jvalue;
	}

	public static void main(String[] args) {
		DistanceMatrix distancematrix = new DistanceMatrix();
		String inputFilename = InOutFileName.getInputFilename();
		distancematrix.readInputCDR(inputFilename);
		;
		distancematrix.process();
		String outputFilename = InOutFileName.getOutputFilename();
		distancematrix.output(outputFilename);
	}

	public boolean readInputCDR(String inputFilename) {
		try {
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(new File(inputFilename))));
			// BufferedReader reader = new BufferedReader(new
			// InputStreamReader(new FileInputStream(new
			// File("S67A_PW99_57_CDR.txt"))));
			String input = null;
			// sequenceMap = new ArrayList<List<String>>();
			names = new ArrayList<String>();
			seqs = new ArrayList<String>();
			vgenes = new ArrayList<String>();
			jgenes = new ArrayList<String>();
			while ((input = reader.readLine()) != null) {

				String[] sequenceList = input.split(",");
				String name = sequenceList[0];
				String vgene = sequenceList[1];
				String jgene = sequenceList[3];
				String sequence = sequenceList[4];
				seqs.add(sequence);
				vgenes.add(vgene);
				jgenes.add(jgene);
				names.add(name);
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	private void process() {
		distanceMatrix = new double[seqs.size()][seqs.size()];
		for (int i = 0; i < seqs.size(); i++) {
			System.out.println(names.get(i));
			String row = seqs.get(i);
			String vrow = vgenes.get(i);
			String jrow = jgenes.get(i);

			for (int j = 0; j < seqs.size(); j++) {
				if (j < i) {
					String col = seqs.get(j);
					String vcol = vgenes.get(j);
					String jcol = jgenes.get(j);
					int[] disVal = NEW_NED(row,col);
/*					System.out.println(disVal[0]);
					System.out.println(disVal[1]);*/
					//distanceMatrix[i][j]= (float)(disVal[0]+VMatrix(vrow, vcol)+JMatrix(jrow, jcol))/disVal[1];
					//distanceMatrix[i][j] = (float) (LD(row,col)+VMatrix(vrow, vcol)+JMatrix(jrow, jcol))/Math.sqrt(Math.max(row.length(), col.length()));
					//distanceMatrix[i][j] = (float) (LD(row,col)+VMatrix(vrow, vcol)+JMatrix(jrow, jcol));
					distanceMatrix[i][j]= (float)(disVal[0])/disVal[1];
					//System.out.println(disVal[1]);
					//distanceMatrix[i][j] = (float) (LD(row,col));
				} else if (j == i) {
					distanceMatrix[i][j] = 0;
				}
			}
		}
		for (int i = 0; i < seqs.size(); i++) {
			for (int j = 0; j < seqs.size(); j++) {
				if (j > i) {
					distanceMatrix[i][j] = distanceMatrix[j][i];
				}
			}
		}
	}

	private void output(String outputFilename) {
		StringBuffer sb = new StringBuffer();
		/*
		 * sb.append("0"); sb.append(","); for(int j=0;j<seqs.size();j++){
		 * sb.append(names.get(j)); if(j<seqs.size()-1){ sb.append(","); }else{
		 * sb.append("\n"); } }
		 */
		for (int i = 0; i < seqs.size(); i++) {

			for (int j = -1; j < seqs.size(); j++) {
				if (j == -1) {
					sb.append(names.get(i));
					sb.append(",");
				} else {
					sb.append(distanceMatrix[i][j]);
					if (j < seqs.size() - 1) {
						sb.append(",");
					} else {
						sb.append("\n");
					}
				}
			}
		}
		try {
			// PrintWriter writer = new PrintWriter(new FileOutputStream(new
			// File("S67A_PW99_57_CDR_dist.csv")));
			PrintWriter writer = new PrintWriter(outputFilename);
			writer.print(sb.toString());
			writer.flush();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

}
