package clustering;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;


/*
 * This is used for handling with R clustering outputs.
 */

public class SortClusterOutput {

	private Map<String, String> seqList;

	private ArrayList<List<String>> ClustFinal;

	public static void main(String[] args) throws Exception {
		SortClusterOutput sortClusterOutput = new SortClusterOutput();
		System.out.println("Please input clustering result first.");
		// 1. Read clustering result as an input.
		String inputFilename = InOutFileName.getInputFilename();
		List<SortCluster> clusters = sortClusterOutput.read(inputFilename);
		/* System.out.println(clusters.size()); */
		// sortClusterOutput.output(clusters, "beforeSort.csv");
		// 2. Sort clustering result with cluster id.
		sortClusterOutput.sortCluster(clusters);
		// sortClusterOutput.output(clusters, "afterSort.csv");

		System.out.println("Please enter CDR3 file name this time.");
		// 3. Read CDR3 file and store as a HashMap.
		String CDR3Filename = InOutFileName.getInputFilename();
		sortClusterOutput.readiHMMune(CDR3Filename);
		// 4. Match sorted clustering result with CDR3 file.
		sortClusterOutput.matchcluster(clusters);
		System.out.println("Please enter the file name for output");
		String outMatchFilename = InOutFileName.getOutputFilename();
		sortClusterOutput.outputMatch(outMatchFilename);
	}

	public void sortCluster(List<SortCluster> clusters) {
		if (clusters == null)
			throw new IllegalArgumentException();
		Collections.sort(clusters, SortCluster.getClusterIdComparator());
	}

	public List<SortCluster> read(String inputFilename) throws Exception {

		/*
		 * BufferedReader reader = new BufferedReader(new InputStreamReader( new
		 * FileInputStream(new File(InOutFileName.getRootPath() +
		 * "/cutYanAndPW.txt"))));
		 */
		// BufferedReader reader = new BufferedReader(new InputStreamReader(new
		// FileInputStream(new File("repertoire/cutYanAndPW.txt"))));
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(new File(inputFilename))));
		String input = null;
		int n = -1;
		List<SortCluster> clusters = new ArrayList<SortCluster>();
		while ((input = reader.readLine()) != null) {
			n++;
			// System.out.println(input);
			if (n == 0)
				continue;

			String seqString = input.substring(0, input.indexOf(" ") + 1);
			seqString = seqString.trim();
			String clusterString = input.substring(input.lastIndexOf(" "));
			clusterString = clusterString.trim();
			// System.out.println(seqString);
			// System.out.println(clusterString);
			int clusterId = -1;
			try {
				clusterId = Integer.parseInt(clusterString);
			} catch (Exception e) {
				e.printStackTrace();
			}
			SortCluster clusterResult = new SortCluster(seqString, clusterId);
			clusters.add(clusterResult);
		}
		return clusters;
	}

	public boolean readiHMMune(String CDR3Filename) {
		try {
			// BufferedReader reader = new BufferedReader(new
			// InputStreamReader(System.in));

			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(new File(CDR3Filename))));
			String input = null;
			// sequenceMap = new ArrayList<List<String>>();
			seqList = new HashMap<String, String>();
			while ((input = reader.readLine()) != null) {

				String[] inputList = input.replace("\"", "").split(",");
				String seqId = inputList[0];

				String seqInfo = inputList[1];
				for (int i = 2; i < inputList.length; i++) {
					seqInfo = seqInfo + "," + inputList[i];
					// System.out.println(seqInfo);
				}
				System.out.println(seqId);
				// System.out.println(seqInfo);
				seqList.put(seqId, seqInfo);
				// System.out.println(seqList.toString());
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	public void output(List<SortCluster> clusters, String filename) {
		File outputFile = new File(filename);
		if (outputFile.exists()) {
			throw new RuntimeException(
					"file already exists, please try another filename for output");
		}
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < clusters.size(); i++) {
			SortCluster result = clusters.get(i);
			sb.append(result.getSequenceId());
			sb.append(",");
			sb.append(String.valueOf(result.getClusterId()));
			if (i < clusters.size() - 1)
				sb.append("\n");
		}

		try {
			PrintWriter writer = new PrintWriter(new FileOutputStream(
					outputFile));
			writer.print(sb.toString());
			writer.flush();
			writer.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void matchcluster(List<SortCluster> clusters) {
		// match sequence id in clustering result with CDR3 and iHMMune-Align
		// result
		ClustFinal = new ArrayList<List<String>>();
		// ArrayList<List<String>> matchedCluster = new
		// ArrayList<List<String>>();
		Map<Integer,Integer> occurs = new TreeMap<Integer,Integer>();
		for (int i = 0; i < clusters.size(); i++) {
			SortCluster clust = clusters.get(i);
			int clusterId = clust.getClusterId();
			Integer occurences = occurs.get(clusterId);
			if(occurences==null){
				occurences = new Integer(1);
				occurs.put(clusterId, occurences);
			}else{
				occurences = new Integer(occurences.intValue()+1);
				occurs.put(clusterId, occurences);
			}
			
		}
		
		
		for (int i = 0; i < clusters.size(); i++) {
			List<String> matchLine = new ArrayList<String>();
			SortCluster clust = clusters.get(i);
			String seqId = clust.getSequenceId();
			matchLine.add(seqId);
			// matchLine.add(",");
			matchLine.add(String.valueOf(clust.getClusterId()));
			String seqInfo = seqList.get(seqId);
			
			String occur = String.valueOf(occurs.get(clust.getClusterId()));
			matchLine.add(occur);
			matchLine.add(seqInfo);
			// System.out.println(matchLine.toString());
			ClustFinal.add(matchLine);
		}
	}

	public void outputMatch(String outMatchFilename) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < ClustFinal.size(); i++) {

			for (int j = 0; j < ClustFinal.get(i).size(); j++) {
				sb.append(ClustFinal.get(i).get(j));
				if (j < (ClustFinal.get(i).size() - 1)) {
					sb.append(",");
				} else {
					sb.append("\n");
				}
			}
			try {
				PrintWriter writer = new PrintWriter(outMatchFilename);
				writer.print(sb.toString());
				writer.flush();
				writer.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}