package com.giantleap.common.nodebean.node.base;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import com.giantleap.common.nodebean.node.base.storage.id.NodeId;
import com.giantleap.common.nodebean.node.base.storage.node.Node;
import com.giantleap.common.nodebean.node.base.storage.node.NodeImp;
import com.giantleap.common.nodebean.node.base.storage.node.Property;
import com.giantleap.common.nodebean.node.base.storage.path.NameInterface;
import com.giantleap.common.nodebean.node.base.storage.value.ArrayListValue;
import com.giantleap.common.nodebean.node.base.storage.value.PropertyType;
import com.giantleap.common.nodebean.node.base.storage.value.StringSetValue;
import com.giantleap.common.nodebean.node.base.storage.value.Value;
import com.giantleap.common.nodebean.node.base.storage.value.ValueFactoryImp;
import com.giantleap.common.nodebean.nodeCollection.NodeCollections;
import com.giantleap.common.util.dataformat.CalendarFormat;


public class ReflectionNode extends NodeImp {
	public static final String NODE_TYPE = "_TP_ReflectionNode";
	public static final String DEFAULT_NODE_NAME = "ReflectionNode";

	//public String _caption;
	
	private boolean enableContextMenu=false;
	
	public ReflectionNode() {
		this(DEFAULT_NODE_NAME);
	}

	public ReflectionNode(String n) {
		this(n, NODE_TYPE);
	}

	// for extended classes
	public ReflectionNode(String n, String nodetype) {
		super(n, nodetype);
	}

	public ReflectionNode(String n,NodeId id,String nodetype){
		super(n, id,nodetype);
	}
	// for valueOf
	public ReflectionNode(NameInterface n, NodeId id) {
		super(n, id);
	}

	public static NodeImp valueOf(NameInterface name, Map<String, Property> properties) {
		if (name == null || properties == null) throw new IllegalArgumentException("null");
		Property idpro = properties.remove(Node.ID_PROPERTY_LITERAL);
		NodeId id = null;
		if (idpro != null) {
			String idString = idpro.getString();
			id = NodeId.valueOf(idString);
		} else {
			id = NodeId.createInstance();
		}
		Property typePro = properties.get(Node.NODE_TYPE_LITERAL);
		String typeStr = null;
		String nameString = null;
		if (typePro == null) {
			nameString = name.getXMLString();
		} else {
			typeStr = typePro.getString().trim();
		}

		String classname = NodeCollections.getJavaClassName(typeStr);

		if (classname == null) {
			// classname =
			// NodeCollections.getJavaClassNameByNodename(nameString);
		}
		if (classname == null) return null;

		NodeImp node = null;
		try {
			Class classDefinition = Class.forName(classname);
			// System.out.println("classname:"+classname);
			Object[] args = new Object[] { name, id };
			Class[] argsClass = new Class[] { NameInterface.class, id.getClass() };
			Constructor constructor = classDefinition.getConstructor(argsClass);
			node = (NodeImp) constructor.newInstance(args);

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		node.setPropertyMap(properties);
		// try {
		Class c = node.getClass();
		Field[] publicFields = c.getDeclaredFields();
		for (Field f : publicFields) {
			String fieldname = f.getName();
			f.setAccessible(true);
			if (fieldname.startsWith("_")) {
				Property pro = properties.get(fieldname);
				if (pro != null) {
					ValueFactoryImp.setField(node, f, pro.getValue());
				}
			}
		}
		return node;
	}

	public void initProperty() {
		Class c = this.getClass();
		Field[] publicFields = c.getDeclaredFields();
		for (Field f : publicFields) {
			String fieldname = f.getName();
			f.setAccessible(true);
			if (fieldname.startsWith("_")) {
				// System.out.println(fieldname);
				updateProperty(fieldname);
			}
		}
	}
	/*
	public static boolean setNodeFieldFromString(ReflectionNode node, String fieldName, String str) {
		Class c = node.getClass();
		Field field = null;
		try {
			field = c.getField(fieldName);
		} catch (Exception e) {
			return false;
		}
		if (field == null) return false;
		return setNodeFieldFromString(node, field, str);
	}*/



	public static Map<String, Value> getNodeFieldsValue(ReflectionNode node) {
		Class c = node.getClass();
		Field[] publicFields = c.getDeclaredFields();
		Map<String, Value> values = new HashMap<String, Value>();
		for (Field f : publicFields) {
			String fieldname = f.getName();
			f.setAccessible(true);
			if (fieldname.startsWith("_")) {
				Value value = ValueFactoryImp.genValue(node, fieldname);
				if (value != null) {
					values.put(fieldname, value);
				}
			}
		}
		return values;
	}

	public static Map<String, String> getFieldsValueString(ReflectionNode node, List<Field> fields) {
		Map<String, Value> values = getFieldsValue(node, fields);
		Map<String, String> strs = new HashMap<String, String>();
		if (values != null) {
			for (Iterator itId = values.entrySet().iterator(); itId.hasNext();) {
				Map.Entry<String, Value> prop = (Map.Entry) itId.next();
				String name = prop.getKey();
				Value value = prop.getValue();
				String valueString = "";
				try {
					valueString = value.getString();
				} catch (Exception e) {
					continue;
				}
				strs.put(name, valueString);
			}
		}
		return strs;
	}

	public static Map<String, Value> getFieldsValue(ReflectionNode node, List<Field> fields) {
		if (node == null || fields == null) return new HashMap<String, Value>();
		Map<String, Value> values = new HashMap<String, Value>();
		for (Field f : fields) {
			String fieldname = f.getName();
			Value value = ValueFactoryImp.genValue(node, fieldname);
			values.put(fieldname, value);
		}
		return values;
	}

	public static Map<String, String> getNodeFieldsString(ReflectionNode node) {
		Map<String, Value> values = getNodeFieldsValue(node);
		Map<String, String> strs = new HashMap<String, String>();
		if (values != null) {
			for (Iterator itId = values.entrySet().iterator(); itId.hasNext();) {
				Map.Entry<String, Value> prop = (Map.Entry) itId.next();
				String name = prop.getKey();
				Value value = prop.getValue();
				String valueString = "";
				try {
					valueString = value.getString();
				} catch (Exception e) {
					continue;
				}
				strs.put(name, valueString);
			}
		}
		return strs;
	}



	public boolean setFields(Map<Field, String> fieldMap) {
		boolean succ = true;
		HashMap<Field, Value> oldvalues = new HashMap<Field, Value>();
		for (Field field : fieldMap.keySet()) {
			Value value = ValueFactoryImp.genValue(this, field);
			if (value != null) {
				oldvalues.put(field, value);
			}
			try {
				succ = ValueFactoryImp.setFieldFromStringViaSetter(this,field,fieldMap.get(field));
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
				succ=false;
			}
			//System.out.println();
			if (!succ)
				break;
		}
		if (!succ) {
			// System.out.println("roll back");//roll back
			for (Field f : oldvalues.keySet()) {
				try {
					ValueFactoryImp.setFieldViaSetter(this, f, oldvalues.get(f));
				} catch (NoSuchMethodException e) {
					e.printStackTrace();
					continue;
				}
			}
			return false;
		}
		return succ;
	}
	


	public static final String JSON_LIST_SEPEROTOR=", ";
	
	private JSONObject getFieldsJSONObject(JSONObject objData,List<Field> fields,String getPropertyNameAction,String listSeperator){
		for (Field f : fields) {
			String fieldname = f.getName();
			String propertyname = fieldname;
			
			Class[] ARGS_CLASS = {String.class};
			Object[] ARGS_OBJECT = {fieldname};
			try {
				propertyname = (String)this.getClass().getMethod(getPropertyNameAction, ARGS_CLASS).invoke(this,ARGS_OBJECT);			
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			try {
				String fieldtype = f.getType().toString();
				int type = PropertyType.valueTypeFromJavaClass(fieldtype);
				if (type == PropertyType.DATE) {
					objData.element(propertyname, CalendarFormat.getCalendarDateString((Calendar) f.get(this)));
				} else if (type == PropertyType.ARRAYLIST_STRING) {
					List<String> list = (List<String>) f.get(this);
					String data = ArrayListValue.listToStr(list,listSeperator);
					objData.element(propertyname,data);
				}else if (type == PropertyType.SET_STRING) {
							Set<String> set = (Set<String>) f.get(this);
							String data = StringSetValue.setToStr(set,listSeperator);
							objData.element(propertyname,data);
				}else {
					String strData = String.valueOf(f.get(this));
					objData.element(propertyname, strData);
				}

			} catch (Exception e) {
				objData.element(propertyname, "");
			}
		}
		return objData;
	}
	
	public List<Field> getIdentityFields(){
		Class c = this.getClass();
		List<Field> fields = new ArrayList<Field>();
		Field[] publicFields = c.getFields();
		for (Field f : publicFields) {
			String fieldname = f.getName();		
			if (fieldname.startsWith("_")) {
				if(fieldname.equalsIgnoreCase("_caption"))continue;
				fields.add(f);
			}
		}
		return fields;
	}
	
	public JSONObject toIdentityJson() {
		JSONObject obj = new JSONObject();
		JSONObject objData = new JSONObject();
		objData = getFieldsJSONObject(objData,getIdentityFields(),"toJsonPropertyName",JSON_LIST_SEPEROTOR);		
		JSONArray ccarray =getIdentityChildrenJsonArray();
		obj.element("name", this.getPath().toString());
		obj.element("data", objData);
		if(ccarray.size()>0){
			obj.element("child", ccarray);
		}
		
		return obj;
	}

	public JSONArray getIdentityChildrenJsonArray(){
		JSONArray ccarray = new JSONArray();
		for (Node cnode : this.getAllChildNode()) {
			if (cnode instanceof ReflectionNode) {
				ReflectionNode rnode=(ReflectionNode)cnode;
				JSONObject cobj = rnode.toIdentityJson();
				ccarray.add(cobj);
			}
		}
		return ccarray;
	}
	
	public String toJsonPropertyName(String name) {
		if (name.length() < 1) return name;
		if (name.startsWith("_")) {
			return name.substring(1);
		} else {
			return name;
		}
	}

	public List<Field> getJSONFields(){
		Class c = this.getClass();
		List<Field> fields = new ArrayList<Field>();
		Field[] publicFields = c.getFields();
		for (Field f : publicFields) {
			String fieldname = f.getName();
			if (fieldname.startsWith("_")) {
				//if(fieldname.equalsIgnoreCase("_caption"))continue;
				fields.add(f);
			}
		}
		return fields;
	}
	
	public boolean setPropertiesByJSON(JSONObject dataObj){
		if(dataObj==null)throw new IllegalArgumentException();
		Map<Field, String> fieldMap = new HashMap<Field, String>();
		List<Field> fields = this.getJSONFields();
		for (Field f : fields) {
			String fieldname = f.getName();
			String fName = toJsonPropertyName(fieldname);
			for (Object key : dataObj.keySet()) {
				String keyString = key.toString();

				if (keyString.equals(fName)) {
					try {
						fieldMap.put(f, URLDecoder.decode((String) dataObj.get(key), "utf-8"));
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}
		return setFields(fieldMap);
	}
	
	public String getJSONCaption(){
		String caption = get_caption();
		if(caption==null||caption.trim().equals("")){
			caption = this.getName().toString();
		}
		return caption;
	}
	
	public JSONObject toJsonObject(String treeid) {
		JSONObject obj = new JSONObject();
		obj.element("id", this.getId().toString());
		if(treeid!=null&&!treeid.trim().equals("")&&!treeid.trim().equals("null")){
			obj.element("treeId",treeid);
		}
		obj.element("caption",getJSONCaption());
		
		JSONObject objData = new JSONObject();
		objData = getFieldsJSONObject(objData,getJSONFields(),"toJsonPropertyName",JSON_LIST_SEPEROTOR);
		obj.element("data", objData);
		
		if(isEnableContextMenu()){
			JSONArray icons     = this.getContextMenuIcons();
			JSONArray captions  = this.getContextMenuCaptions();
			JSONArray seperators= this.getContextMenuSeperators();
			JSONArray disabled  = this.getContextMenuDisabled();
			JSONObject actions  = this.getContextMenuActions();			
			JSONObject contextMenu = new JSONObject();
			contextMenu.element("captions", captions);
			if(icons.size()>0){
				contextMenu.element("icons", icons);
			}	
			if(seperators.size()>0){
				contextMenu.element("seperators", seperators);
			}
			if(disabled.size()>0){
				contextMenu.element("disabled", disabled);
			}
			contextMenu.element("actions", actions);
			obj.element("contextMenuOptions", contextMenu);
		}
		JSONArray ccarray =getChildrenJsonArray(treeid);
		if(ccarray.size()>0){
			obj.element("child", ccarray);
		}
		
		return obj;
	}
	
	public JSONArray getChildrenJsonArray(String treeid){
		JSONArray ccarray = new JSONArray();
		for (Node cnode : this.getAllChildNode()) {
			if (cnode instanceof ReflectionNode) {
				ReflectionNode rnode=(ReflectionNode)cnode;
				JSONObject cobj = rnode.toJsonObject(treeid);
				ccarray.add(cobj);
			}
		}
		return ccarray;
	}
	
	public JSONArray getLogicChildrenJsonArray(String treeid){
		JSONArray ccarray = new JSONArray();
		for (Node cnode : this.getAllLogicChildNode()) {
			if (cnode instanceof ReflectionNode) {
				ReflectionNode rnode=(ReflectionNode)cnode;
				JSONObject cobj = rnode.toJsonObject(treeid);
				ccarray.add(cobj);
			}
		}
		return ccarray;
	}
	/*
	public JSONArray getLogicChildrenJsonArray(String treeid){
		JSONArray ccarray = new JSONArray();
		for (Node cnode : this.getAllLogicChildNode()) {
			if (cnode instanceof ReflectionNode) {
				ReflectionNode rnode=(ReflectionNode)cnode;
				JSONObject cobj = rnode.toJsonObject(treeid);
				ccarray.add(cobj);
			}
		}
		return ccarray;
	}*/
	
	protected JSONArray getContextMenuIcons(){
		JSONArray icons = new JSONArray();
		icons.add("edit");
		icons.add("add");
		icons.add("delete");
		return icons;
	}
	
	protected JSONArray getContextMenuCaptions(){
		JSONArray captions = new JSONArray();
		captions.add("Edit");
		captions.add("Add New Node");
		captions.add("Delete");
		return captions;
	}
	
	protected JSONArray getContextMenuSeperators(){
		JSONArray seperators = new JSONArray();
		seperators.add(1);
		seperators.add(2);
		return seperators;
	}
	
	protected JSONArray getContextMenuDisabled(){
		JSONArray disabled = new JSONArray();
		return disabled;
	}
	protected JSONObject getContextMenuActions(){
		JSONObject actions = new JSONObject();
		return actions;
	}
	
	public boolean isEnableContextMenu() {
		return enableContextMenu;
	}
	public void setEnableContextMenu(boolean enableContextMenu) {
		this.enableContextMenu = enableContextMenu;
	}
	

	

	
	public String get_caption() {
		return this.getName().toString();
	}


	public String getType() {
		return NODE_TYPE;
	}
}
