package com.giantleap.common.nodebean.node.base.filter;

import com.giantleap.common.nodebean.node.base.storage.node.Node;



public class AndFilter implements NodeBeanFilter{
	protected NodeBeanFilter[] _predicates;

	/**
	 * Creates a new instance of an AndFilter. With no predicates, this would
	 * always answer <code>true</code> to {@link #accept}.
	 * 
	 * @see #setPredicates
	 */
	public AndFilter() {
		setPredicates(null);
	}

	/**
	 * Creates an AndFilter that accepts nodes acceptable to both filters.
	 * 
	 * @param left
	 *            One filter.
	 * @param right
	 *            The other filter.
	 */
	public AndFilter(NodeBeanFilter left, NodeBeanFilter right) {
		NodeBeanFilter[] predicates;

		predicates = new NodeBeanFilter[2];
		predicates[0] = left;
		predicates[1] = right;
		setPredicates(predicates);
	}

	/**
	 * Creates an AndFilter that accepts nodes acceptable to all given filters.
	 * 
	 * @param predicates
	 *            The list of filters.
	 */
	public AndFilter(NodeBeanFilter[] predicates) {
		setPredicates(predicates);
	}

	/**
	 * Get the predicates used by this AndFilter.
	 * 
	 * @return The predicates currently in use.
	 */
	public NodeBeanFilter[] getPredicates() {
		return (_predicates);
	}

	/**
	 * Set the predicates for this AndFilter.
	 * 
	 * @param predicates
	 *            The list of predidcates to use in {@link #accept}.
	 */
	public void setPredicates(NodeBeanFilter[] predicates) {
		if (null == predicates)
			predicates = new NodeBeanFilter[0];
		_predicates = predicates;
	}

	//
	// NodeFilter interface
	//

	/**
	 * Accept nodes that are acceptable to all of its predicate filters.
	 * 
	 * @param node
	 *            The node to check.
	 * @return <code>true</code> if all the predicate filters find the node is
	 *         acceptable, <code>false</code> otherwise.
	 */
	public boolean accept(Node node) {
		boolean ret;

		ret = true;

		for (int i = 0; ret && (i < _predicates.length); i++)
			if (!_predicates[i].accept(node))
				ret = false;

		return (ret);
	}
}
