package com.giantleap.common.nodebean.node.base.filter;

import com.giantleap.common.nodebean.node.base.storage.node.Node;



public class OrFilter implements NodeBeanFilter{
	protected NodeBeanFilter[] _predicates;

	/**
	 * Creates a new instance of an OrFilter. With no predicates, this would
	 * always answer <code>false</code> to {@link #accept}.
	 * 
	 * @see #setPredicates
	 */
	public OrFilter() {
		setPredicates(null);
	}

	/**
	 * Creates an OrFilter that accepts nodes acceptable to either filter.
	 * 
	 * @param left
	 *            One filter.
	 * @param right
	 *            The other filter.
	 */
	public OrFilter(NodeBeanFilter left, NodeBeanFilter right) {
		NodeBeanFilter[] predicates;

		predicates = new NodeBeanFilter[2];
		predicates[0] = left;
		predicates[1] = right;
		setPredicates(predicates);
	}

	/**
	 * Creates an OrFilter that accepts nodes acceptable to any of the given
	 * filters.
	 * 
	 * @param predicates
	 *            The list of filters.
	 */
	public OrFilter(NodeBeanFilter[] predicates) {
		setPredicates(predicates);
	}

	/**
	 * Get the predicates used by this OrFilter.
	 * 
	 * @return The predicates currently in use.
	 */
	public NodeBeanFilter[] getPredicates() {
		return (_predicates);
	}

	/**
	 * Set the predicates for this OrFilter.
	 * 
	 * @param predicates
	 *            The list of predidcates to use in {@link #accept}.
	 */
	public void setPredicates(NodeBeanFilter[] predicates) {
		if (null == predicates)
			predicates = new NodeBeanFilter[0];
		_predicates = predicates;
	}

	//
	// NodeFilter interface
	//

	/**
	 * Accept nodes that are acceptable to any of its predicate filters.
	 * 
	 * @param node
	 *            The node to check.
	 * @return <code>true</code> if any of the predicate filters find the node
	 *         is acceptable, <code>false</code> otherwise.
	 */
	public boolean accept(Node node) {
		boolean ret;

		ret = false;

		for (int i = 0; !ret && (i < _predicates.length); i++)
			if (_predicates[i].accept(node))
				ret = true;

		return (ret);
	}
}
