package com.giantleap.common.nodebean.node.base.storage.id;


public class Base64 {

	public static final int UUID_BYTE_LENGTH = 16;

	public static final int BYTE_BITS_LENGTH = 8;
	public static final int CODE_BITS_LENGTH = 6;

	// "final" the best; I just don't want to type...
	public static char[] CHARSET = null;
	public static int[] REVERSE_CHARSET = null;
	public static void init() {
		CHARSET = new char[64];
		REVERSE_CHARSET = new int[128];
		
		int i = 0;
		for (char c = '0'; c <= '9'; c++, i++){
			CHARSET[i] = c;
			REVERSE_CHARSET[c] = i;		
		}

		for (char c = 'A'; c <= 'Z'; c++, i++){
			CHARSET[i] = c;
			REVERSE_CHARSET[c] = i;			
		}

		for (char c = 'a'; c <= 'z'; c++, i++){
			CHARSET[i] = c;
			REVERSE_CHARSET[c] = i;
		}
			

		CHARSET[i] = '.';
		CHARSET[i + 1] = '-';
		
		REVERSE_CHARSET['.'] = i;
		REVERSE_CHARSET['-'] = i + 1;
	}

	public static String encode(byte[] data) throws IllegalArgumentException {
		if (CHARSET == null)
			init();

		if (data.length != UUID_BYTE_LENGTH)
			throw new IllegalArgumentException(
					"UUID must be constructed using a 16 byte array.");

		StringBuffer sb=new StringBuffer();
		//String strBuffer = "";

		int top = 0;
		int buffer = 0;

		for (int i = UUID_BYTE_LENGTH - 1; i >= 0; i--) {
			buffer = buffer | ((data[i] & 0xFF) << top);
			top += BYTE_BITS_LENGTH;

			while (top > CODE_BITS_LENGTH) {
				sb.append(CHARSET[buffer & (CHARSET.length - 1)]);
				//strBuffer = CHARSET[buffer & (CHARSET.length - 1)] + strBuffer;
				buffer >>>= CODE_BITS_LENGTH;
				top -= CODE_BITS_LENGTH;
			}
		}

		if (top > 0){
			//strBuffer = CHARSET[buffer & (CHARSET.length - 1)] + strBuffer;
			sb.append(CHARSET[buffer & (CHARSET.length - 1)]);
		}

		return sb.toString();
	}
	
	
	public static String encode(NodeId id) throws IllegalArgumentException {
		UUID uuid=id.getUUID();
		byte[] bytes=uuid.getRawBytes();
		return encode(bytes);
	}
	
	public static String encode(String id) throws IllegalArgumentException {
		NodeId nodeid=NodeId.valueOf(id);
		UUID uuid=nodeid.getUUID();
		byte[] bytes=uuid.getRawBytes();
		return encode(bytes);
	}
	
	public static byte[] decode(String data) throws IllegalArgumentException {
		if (CHARSET == null) init();

		//int standardLength = UUID_BYTE_LENGTH * BYTE_BITS_LENGTH / CODE_BITS_LENGTH + 1;
		int standardLength = (int) Math.ceil(UUID_BYTE_LENGTH * BYTE_BITS_LENGTH / (double) CODE_BITS_LENGTH);
		if ((data == null) || (data.length() != standardLength))
			throw new IllegalArgumentException("Code should be a "+ standardLength + " chars string.");

		int top = 0;
		int buffer = 0;

		int index = UUID_BYTE_LENGTH - 1;
		byte[] result = new byte[UUID_BYTE_LENGTH];

		for (int i = standardLength - 1; i >= 0; i--) {
			buffer |= REVERSE_CHARSET[data.charAt(i)] << top;
			top += CODE_BITS_LENGTH;

			while (top > BYTE_BITS_LENGTH) {
				result[index] = (byte) (buffer & ((1 << BYTE_BITS_LENGTH) - 1));
				buffer >>= BYTE_BITS_LENGTH;
				top -= BYTE_BITS_LENGTH;
				index--;
			}
		}

		if (top > BYTE_BITS_LENGTH) {
			result[index] = (byte) (buffer & ((1 << BYTE_BITS_LENGTH) - 1));
		}

		return result;
	}
	
	/**
	 * test
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		byte[] id = new byte[16];
		for (int i = 0; i < 16; i++) {
			id[i] = (byte) ((Math.random() - 0.5) * 2 * 128);
		}

		//System.out.println(Arrays.toString(id));
		//System.out.println(Base64.encode(id));
		
		UUID uuid=UUID.randomUUID();
		byte[] bytes=uuid.getRawBytes();
		
		System.out.println(uuid.toString());
		System.out.println(Base64.encode(bytes));
	}
}
