package com.giantleap.common.nodebean.node.base.storage.id;





/**
 * Node identifier. An instance of this class identifies a node using its UUID.
 * Once created a node identifier instance is immutable.
 */
public class NodeId {

    /** Serial version UID of this class. */
    static final long serialVersionUID = 7380115476447060008L;

    /** UUID of the identified node */
    private final UUID uuid;

    /** the precalculated hashcode */
    private final int hashCode;
    
    

    /**
     * Creates a node identifier instance for the identified node.
     *
     * @param uuid node UUID
     */
    public NodeId(UUID uuid) {
        if (uuid == null) {
            throw new IllegalArgumentException("uuid can not be null");
        }
        this.uuid = uuid;
        this.hashCode = uuid.hashCode();
    }

    
    public static NodeId createInstance(){
    	UUID uuid = UUID.randomUUID();
    	return new NodeId(uuid);
    }
    /**
     * Returns <code>true</code> as this class represents a node identifier,
     * not a property identifier.
     *
     * @return always <code>true</code>
     * @see ItemId#denotesNode()
     */
    public boolean denotesNode() {
        return true;
    }

    /**
     * Returns the UUID of the identified node.
     *
     * @return node UUID
     */
    public UUID getUUID() {
        return uuid;
    }

    /**
     * Returns a <code>NodeId</code> holding the value of the specified
     * string. The string must be in the format returned by the
     * <code>NodeId.toString()</code> method.
     *
     * @param s a <code>String</code> containing the <code>NodeId</code>
     *          representation to be parsed.
     * @return the <code>NodeId</code> represented by the argument
     * @throws IllegalArgumentException if the specified string can not be parsed
     *                                  as a <code>NodeId</code>.
     * @see #toString()
     */
    public static NodeId valueOf(String s) throws IllegalArgumentException {
        if (s == null) {
            throw new IllegalArgumentException("invalid NodeId literal");
        }
        //return  NodeId.createInstance();
        return new NodeId(new UUID(s));
    }

    //-------------------------------------------< java.lang.Object overrides >
    /**
     * {@inheritDoc}
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof NodeId) {
            return uuid.equals(((NodeId) obj).uuid);
        }
        return false;
    }

    /**
     * {@inheritDoc}
     *
     * Returns the same as <code>this.getUUID().toString()</code>
     */
    public String toString() {
        return uuid.toString();
    }

    /**
     * {@inheritDoc}
     *
     * Returns the same as <code>this.getUUID().hashCode()</code>
     */
    public int hashCode() {
        return hashCode;
    }
}
