package com.giantleap.common.nodebean.node.base.storage.node;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.collections.map.LinkedMap;

import com.giantleap.common.nodebean.node.base.filter.NodeBeanFilter;
import com.giantleap.common.nodebean.node.base.storage.id.NodeId;
import com.giantleap.common.nodebean.node.base.storage.node.exception.ItemExistsException;
import com.giantleap.common.nodebean.node.base.storage.node.exception.ItemNotFoundException;
import com.giantleap.common.nodebean.node.base.storage.path.NameInterface;
import com.giantleap.common.nodebean.node.base.storage.path.PathElement;
import com.giantleap.common.nodebean.node.base.storage.path.PathInterface;
import com.giantleap.common.nodebean.node.base.storage.value.IllegalValueException;
import com.giantleap.common.nodebean.node.base.storage.value.Value;


public interface Node extends Item{
	public boolean hasChild(Node node);
	public boolean hasLogicChild(Node node);
	
	public LinkedMap getChildNodes();
	public LinkedMap getLogicChildNodes();
	
    //public Collection<Node> getAllChildNodeNonExtending();
    public Collection<Node> getAllChildNode();
    public Collection<Node> getAllLogicChildNode();
	
	public Map<NameInterface, ArrayList<NodeId>> getChildEntries();
	public Map<NameInterface, ArrayList<NodeId>> getLogicChildEntries();
	
    public List<Node> getChildNodeList(String name);
    public List<Node> getChildNodeList(NameInterface name);
    public List<Node> getLogicChildNodeList(String name);
    public List<Node> getLogicChildNodeList(NameInterface name);
	
	public int getChildNameCount(String namestr);
    public int getChildNameCount(NameInterface name);
    public int getLogicChildNameCount(String namestr);
    public int getLogicChildNameCount(NameInterface name);
    
    public Node getNode(String id);
    public Node getLogicNode(String id);
    
    public Node getChildNode(String id);
    public Node getChildNode(NodeId id);
    public Node getLogicChildNode(String id);
    public Node getLogicChildNode(NodeId id);
    
    public Node getChildNode(PathElement e);
    public Node getChildNode(NameInterface name, int index);
    public Node getLogicChildNode(PathElement e);
    public Node getLogicChildNode(NameInterface name, int index);
    
    public Node getNodeByPath(String relpathStr);
    public Node getNodeByPath(PathInterface relpath);
    public Node getLogicNodeByPath(String relpathStr);
    public Node getLogicNodeByPath(PathInterface relpath);
    
    public Node getLogicNodeByPath(Node node,PathInterface relpath);
    
    public boolean isRoot();
    public boolean isLogicRoot();
    //public int getChildIndex(Node node)throws ItemNotFoundException;
    //public int getChildIndex(NodeId childId)throws ItemNotFoundException;
	
    public Node addChildNode(Node childNode);
    
    public Node removeChildNode(Node node,boolean recursive);
    public Node removeChildNode(NameInterface name,int index);
    public List<Node> removeChildNodesByName(NameInterface name);
    public void removeAllChildNodes();

    public List<Node> getParentNodeList();
    public List<Node> getParentNodeList(boolean reverse);
    //public Node getChildNode(String id);
    //public Node getChildNode(NodeId id);
    //public Node getChildNode(PathElement e);
    //public Node getChildNode(NameInterface name, int index);
   // public Node getNode(String id);
    //public Node getNodeByPath(String relpathStr);
    //public Node getNodeByPath(PathInterface relpath);
    
    public boolean exchangeOrder(Node node1,Node node2);
    public boolean orderBefore(Node node,int destPos);
    //public boolean orderBefore(String childname, int destIndex,int srcIndex)throws ItemNotFoundException;
    //public boolean orderBefore(NameInterface childname, int destIndex,int srcIndex)throws ItemNotFoundException;

    public Property getProperty(String name);
    public Property addProperty(String name, Property property)throws ItemExistsException;
    public Property setOrAddProperty(String name, Property prop);
    public Property setOrAddProperty(String name, String proValue,String proType) throws IllegalValueException;
    public Property setOrAddProperty(String name, String proValue,int proType) throws IllegalValueException;
    public Property setOrAddProperty(String name, Value value) throws IllegalValueException;
    public Property removeProperty(String name)throws ItemNotFoundException;
    public Map<String,Property> getPropertyMap();
    public void setPropertyMap(Map properties);
    
    
    public NodeId getId();
    public String getIdString();
    public void newId(boolean recursive);
    



    //public boolean addPersistentChildNode(Node childNode)throws IllegalArgumentException,ItemExistsException;
    //public Node addIsolatedChildNode(String nameString);
    //public Node addIsolatedChildNode(NameInterface childName);
    //public Node addIsolatedChildNode(Node childNode);
    //public void setId(String id);
    //public void setPropertyMap(Map properties);

    //public Property removeIdProperty();
    //public Property addProperty(String name, Property property)throws ItemExistsException,InvalidItemStateException;
    
    //public LinkedMap getChildNodes();
    
    //public HashMap<NameInterface, ArrayList<NodeId>> getChildEntries();
    public void setChildNodes(LinkedMap map);
    public void setChildEntries(HashMap<NameInterface, ArrayList<NodeId>> entries);
    
    public Node getLogicPeer();
    public boolean equalsRecursive(Object obj);
    //public boolean isRoot();
    public String getType();

    
    //public void init(boolean recursive);
    public List<Node> extract(NodeBeanFilter filter,boolean recursive);
    public List<Node> extractExtended(NodeBeanFilter filter);
    public List<Node> extractStruct(NodeBeanFilter filter,boolean recursive);
    //public List<Node> extractStructDeepFirst(NodeBeanFilter filter);
    
    public boolean fireBeforeRemoveChild();
    public boolean fireAfterRemoveChild();
	
    public boolean fireBeforeBeRemoved();
    public boolean fireAfterBeRemoved();
    
    public boolean fireBeforeAddChild();
    public boolean fireAfterAddChild();
	
    public boolean fireBeforeBeAdded();
    public boolean fireAfterBeAdded();
    
    public void updateChildPath(boolean recursive,boolean extend);
    public JSONObject toIdentityJson();
    public JSONArray getIdentityChildrenJsonArray();
    
    public static final String NODE_TYPE_LITERAL = "_TYPE_";
    public static String ID_PROPERTY_LITERAL="_ID_";
}
