package com.giantleap.common.nodebean.node.base.storage.node;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.collections.map.LinkedMap;

import com.giantleap.common.exception.GLException;
import com.giantleap.common.nodebean.node.base.filter.NodeBeanFilter;
import com.giantleap.common.nodebean.node.base.storage.id.NodeId;
import com.giantleap.common.nodebean.node.base.storage.node.exception.ItemExistsException;
import com.giantleap.common.nodebean.node.base.storage.node.exception.ItemNotFoundException;
import com.giantleap.common.nodebean.node.base.storage.path.NameImp;
import com.giantleap.common.nodebean.node.base.storage.path.NameInterface;
import com.giantleap.common.nodebean.node.base.storage.path.PathElement;
import com.giantleap.common.nodebean.node.base.storage.path.PathImp;
import com.giantleap.common.nodebean.node.base.storage.path.PathInterface;
import com.giantleap.common.nodebean.node.base.storage.path.exception.IllegalNameException;
import com.giantleap.common.nodebean.node.base.storage.value.IllegalValueException;
import com.giantleap.common.nodebean.node.base.storage.value.PropertyType;
import com.giantleap.common.nodebean.node.base.storage.value.Value;
import com.giantleap.common.nodebean.node.base.storage.value.ValueFactoryImp;





public class NodeImp extends ItemImp implements Node{
	private LinkedMap _childNodes  = new LinkedMap();
	private HashMap<NameInterface,ArrayList<NodeId>>   _childEntries= new HashMap<NameInterface,ArrayList<NodeId>>();

	private Map<String,Property> _properties = new HashMap<String,Property>();
	//readonly CAN NOT be null;
	private NodeId  _nodeId=null;
	public static final String NODE_TYPE = "_TP_NM_";
	
	protected NodeImp(NameInterface name,String type){
		super(name);
		_nodeId=NodeId.createInstance();			
		try {
			Property property = genIdProperty(_nodeId);
			if(property!=null)this.addProperty(Node.ID_PROPERTY_LITERAL, property);
			this.setOrAddProperty(Node.NODE_TYPE_LITERAL, type,PropertyType.STRING);
		} catch (GLException e) {
			e.printStackTrace();
			//throw new RuntimeException();
		}
	}
	
	protected NodeImp(String name,String type){
		super(name);
		_nodeId=NodeId.createInstance();
		try {
			Property property = genIdProperty(_nodeId);
			if(property!=null)this.addProperty(Node.ID_PROPERTY_LITERAL, property);
			this.setOrAddProperty(Node.NODE_TYPE_LITERAL, type,PropertyType.STRING);
		} catch (GLException e) {
			e.printStackTrace();
		}
	}

	protected NodeImp(String n,NodeId id,String type){
		super(n);
		if(id!=null){
			_nodeId=id;
		}else{
			_nodeId=NodeId.createInstance();
		}
		try {
			Property property = genIdProperty(_nodeId);
			if(property!=null)this.addProperty(Node.ID_PROPERTY_LITERAL, property);
			this.setOrAddProperty(Node.NODE_TYPE_LITERAL, type,PropertyType.STRING);
		} catch (GLException e) {
			e.printStackTrace();
			//throw new RuntimeException();
		}
	}
	
	protected NodeImp(NameInterface n,NodeId id){
		super(n);
		if(id!=null){
			_nodeId=id;
		}else{
			_nodeId=NodeId.createInstance();
		}
		try {
			Property property = genIdProperty(_nodeId);
			if(property!=null)this.addProperty(Node.ID_PROPERTY_LITERAL, property);
		} catch (GLException e) {
			e.printStackTrace();
			//throw new RuntimeException();
		}
	}
	
	public void makeLogicRoot(){
		this.setPath(PathInterface.ROOT_PATH);
	}
	public Collection<Node> getAllChildNode(){
		return getChildNodes().values();
	}
	/*
	public Collection<Node> getAllChildNodeNonExtending(){
		return getAllChildNode();
	}*/
	
	
	public Collection<Node> getAllLogicChildNode(){
		return getLogicChildNodes().values();
	}
	
	
	


	public boolean hasChild(Node node){
		if(node==null)return false;
		LinkedMap cMap=getChildNodes();
		return cMap.containsKey(node.getId());
	}
	
	public boolean hasLogicChild(Node node){
		if(node==null)return false;
		LinkedMap cMap=getLogicChildNodes();
		return cMap.containsKey(node.getId());
	}
	
	private void setChildEntry(NameInterface name,ArrayList<NodeId> list){
		getChildEntries().put(name, list);
	}
	

	public int getChildNameCount(String namestr){
		if(namestr==null) throw new IllegalArgumentException("null");
		NameInterface n=null;
		try {
			n=NameImp.createName(namestr);
		} catch (IllegalNameException e) {
			return 0;
		}
		return getChildNameCount(n);
	}
	public int getChildNameCount(NameInterface name){
		if(name==null) throw new IllegalArgumentException("null");
		ArrayList<NodeId> list = getChildEntries().get(name);
        if (list == null) return 0;
        return list.size();
	}
	public int getLogicChildNameCount(String namestr){
		if(namestr==null) throw new IllegalArgumentException("null");
		NameInterface n=null;
		try {
			n=NameImp.createName(namestr);
		} catch (IllegalNameException e) {
			return 0;
		}
		return getLogicChildNameCount(n);
	}
	public int getLogicChildNameCount(NameInterface name){
		if(name==null) throw new IllegalArgumentException("null");
		ArrayList<NodeId> list = getLogicChildEntries().get(name);
        if (list == null) return 0;
        return list.size();
	}

	public List<Node> getChildNodeList(String name){
		if(name==null) throw new IllegalArgumentException("null");
		NameInterface n=null;
		try{
			n=NameImp.createName(name);
		}catch(GLException e){
			throw new IllegalArgumentException();
		}
		return getChildNodeList(n);
	}
	
	public List<Node> getChildNodeList(NameInterface name){
		if(name==null) throw new IllegalArgumentException("null");
		ArrayList<Node> nodelist=new ArrayList<Node>();
		ArrayList<NodeId> list=getChildEntries().get(name);
		if(list==null){
			return nodelist;
		}
		
		if(list.size()==0) return nodelist;
		for(int i=0;i<list.size();i++){
			Node node=(Node)getChildNodes().get(list.get(i)); 
			if(node!=null)nodelist.add(node);
		}
		return nodelist;
	}
	
	public List<Node> getLogicChildNodeList(String name){
		if(name==null) throw new IllegalArgumentException("null");
		NameInterface n=null;
		try{
			n=NameImp.createName(name);
		}catch(GLException e){
			throw new IllegalArgumentException();
		}
		return getLogicChildNodeList(n);
	}
	
	public List<Node> getLogicChildNodeList(NameInterface name){
		if(name==null) throw new IllegalArgumentException("null");
		ArrayList<Node> nodelist=new ArrayList<Node>();
		ArrayList<NodeId> list=getLogicChildEntries().get(name);
		if(list==null){
			return nodelist;
		}
		
		if(list.size()==0) return nodelist;
		for(int i=0;i<list.size();i++){
			Node node=(Node)getLogicChildNodes().get(list.get(i)); 
			if(node!=null)nodelist.add(node);
		}
		return nodelist;
	}
	
	public Node getNode(String id){
		NodeId nodeid = null;
		try{
			nodeid = NodeId.valueOf(id);
		}catch(Exception e){
			return null;
		}
		if(this.getId().equals(nodeid))return this;
		return getNode(this,nodeid);
	}
	
	private static Node getNode(Node node,NodeId id){
		if(id.equals(node.getId())) return node;
		for(Node child:node.getAllChildNode()){
			Node cnode=getNode(child,id);
			if(cnode!=null){
				return cnode;
			}
		}
		return null;
	}
	
	public Node getLogicNode(String id){
		NodeId nodeid = null;
		try{
			nodeid = NodeId.valueOf(id);
		}catch(Exception e){
			return null;
		}
		if(this.getId().equals(nodeid))return this;
		return getLogicNode(this,nodeid);
	}
	
	private static Node getLogicNode(Node node,NodeId id){
		if(id.equals(node.getId())) return node;
		for(Node child:node.getAllLogicChildNode()){
			Node cnode=getNode(child,id);
			if(cnode!=null){
				return cnode;
			}
		}
		return null;
	}
	
	public Node getChildNode(String id){
		NodeId nodeid = null;
		try{
			nodeid = NodeId.valueOf(id);
		}catch(Exception e){
			return null;
		}
		return getChildNode(nodeid);	
	}
	
	public Node getChildNode(NodeId id){
		if(id==null) return null;
		return (Node)getChildNodes().get(id);
	}
	
	public Node getLogicChildNode(String id){
		NodeId nodeid = null;
		try{
			nodeid = NodeId.valueOf(id);
		}catch(Exception e){
			return null;
		}
		return getLogicChildNode(nodeid);	
	}
	
	public Node getLogicChildNode(NodeId id){
		if(id==null) return null;
		return (Node)getLogicChildNodes().get(id);
	}
	
	public Node getChildNode(PathElement e){
		if(e==null) throw new IllegalArgumentException("invalid element");		
		Node found=getChildNode(e.getName(),e.getIndex());
		return found;
	}
	
	public Node getChildNode(NameInterface name, int index){
		if(name==null||index<0) return null;
        List<NodeId> list = getChildEntries().get(name);
        //System.out.println(this.getPath());
        //System.out.println(this.getAllChildNode().size());
        if (list == null) return null;
        if(index>list.size()) return null;
        NodeId id=(NodeId)list.get(index-PathElement.INDEX_DEFAULT); 
        return getChildNode(id);
	}
	
	public Node getLogicChildNode(PathElement e){
		if(e==null) throw new IllegalArgumentException("invalid element");		
		Node found=getLogicChildNode(e.getName(),e.getIndex());
		return found;
	}
	
	public Node getLogicChildNode(NameInterface name, int index){
		if(name==null||index<0) return null;
        List<NodeId> list = getLogicChildEntries().get(name);
        //System.out.println(this.getPath());
        //System.out.println(this.getAllChildNode().size());
        if (list == null) return null;
        if(index>list.size()) return null;
        NodeId id=(NodeId)list.get(index-PathElement.INDEX_DEFAULT); 
        return getLogicChildNode(id);
	}
	
	public Node getNodeByPath(String relpathStr){
		if(relpathStr==null) throw new IllegalArgumentException("null"); 
		PathInterface path=null;
		try{
			path=PathImp.createPath(relpathStr);
		}catch(Exception e){
		}
		//System.out.println("path:"+path);
		return this.getNodeByPath(path);
	}
	
	public Node getNodeByPath(PathInterface relpath){
		if(relpath==null) return null;
		return getNodeByPath(this,relpath);
	}

	public static Node getNodeByPath(Node node,PathInterface relpath){
		if(node==null) throw new IllegalArgumentException("null node");
		if(relpath==null) return null;
		if(relpath.isAbsolute()){
			relpath=PathImp.getChildPath(relpath);
		}
		try{
			if(!relpath.isCanonical()) relpath=relpath.getNormalizedPath();
		}catch(GLException e){
			return null;
		}		
		
		PathElement ele=relpath.getFirstElement();
		if(ele==null) return null;
		//System.out.println("getPath:"+ele.toString());
		Node childNode=node.getChildNode(ele);
		
		if(childNode==null){
			return null;
		}
		
		PathInterface childRelPath = PathImp.getChildPath(relpath);
		if(childRelPath!=null){
			Node found=getNodeByPath(childNode,childRelPath);
			return found;
		}else{
			return childNode;
		}
	}
	
	public Node getLogicNodeByPath(String relpathStr){
		if(relpathStr==null) throw new IllegalArgumentException("null"); 
		PathInterface path=null;
		try{
			path=PathImp.createPath(relpathStr);
		}catch(Exception e){
		}
		//System.out.println("path:"+path);
		return this.getLogicNodeByPath(path);
	}
	
	public Node getLogicNodeByPath(PathInterface relpath){
		if(relpath==null) return null;
		return getLogicNodeByPath(this,relpath);
	}

	public Node getLogicNodeByPath(Node node,PathInterface relpath){
		if(node==null) throw new IllegalArgumentException("null node");
		if(relpath==null) return null;
		if(relpath.isAbsolute()){
			relpath=PathImp.getChildPath(relpath);
			//System.out.println("abs");
		}
		try{
			if(!relpath.isCanonical()) relpath=relpath.getNormalizedPath();
		}catch(GLException e){
			return null;
		}		
		
		PathElement ele=relpath.getFirstElement();
		if(ele==null) return null;
		//System.out.println("getPath:"+ele.toString());
		Node childNode=node.getLogicChildNode(ele);
		//System.out.println("childNode:"+childNode.getClass());
		if(childNode==null){
			return null;
		}
		
		PathInterface childRelPath = PathImp.getChildPath(relpath);
		if(childRelPath!=null){
			Node found=getLogicNodeByPath(childNode,childRelPath);
			return found;
		}else{
			return childNode;
		}
	}
	/*
	public int getChildIndex(Node node)throws ItemNotFoundException{
		return getChildIndex(node.getId());
	}
	
	public int getChildIndex(NodeId childId)throws ItemNotFoundException{
		if(childId==null) throw new IllegalArgumentException("null");
    	Node node=(Node)getChildNodes().get(childId);
    	if(node==null) throw new ItemNotFoundException("nodeId:"+childId);
		NameInterface childName=node.getName();	
		ArrayList<NodeId> list = getChildEntries().get(childName);
        if (list == null) throw new ItemNotFoundException("name="+childName);;
        return list.indexOf(childId);
	}
	*/
	
	public Node addChildNode(Node childNode){
    	if(childNode==null) throw new IllegalArgumentException("null");
    	childNode=internalAddChildNode(childNode);        
        return childNode;
    }
	
	private Node internalAddChildNode(Node childNode){
		if(childNode==null) throw new IllegalArgumentException("null");
		NameInterface nodeName = childNode.getName();
		//System.out.println(nodeName);
    	if(nodeName==null) throw new IllegalArgumentException("null");
    	NodeId id=childNode.getId();
    	if(id!=null){
        	Node node=(Node)getChildNodes().get(id);
        	if(node!=null) throw new IllegalArgumentException("nodeId:"+id+" Node already Exists");       		
    	}
    	
    	if(!this.fireBeforeAddChild()) return childNode;
    	if(!childNode.fireBeforeBeAdded())return childNode;
    	
    	childNode.setParent(this);
    	childNode.setPath(this);
    	
        ArrayList<NodeId> list = getChildEntries().get(nodeName);
        if (list != null) {
        	list.add(id);                	
        } else {
                // convert to siblings list
        	list = new ArrayList<NodeId>();
        	list.add(id);
        	getChildEntries().put(nodeName, list);
        }
        getChildNodes().put(id,childNode);
        
        childNode.updateChildPath(true, false);
        
        this.fireAfterAddChild();
        childNode.fireAfterBeAdded();
        return childNode;
	}
	
	public Item setParent(Node pnode){
		parent=pnode;
		return this;
	}
	
	public PathInterface setPath(Node parent){
		if(parent==null)  throw new IllegalArgumentException("null");
		int index=parent.getLogicChildNameCount(getName());
		PathInterface nodepath=null;
		try{
			PathInterface parentPath=parent.getPath();
			//System.out.println(parent.getId());
			//System.out.println(getName());
			
			//System.out.println(parentPath);
			nodepath=PathImp.createPath(parentPath,getName(),index+PathElement.INDEX_DEFAULT,true);				
			if(nodepath!=null){
				setPath(nodepath);
				//System.out.println(nodepath);
			}
		}catch(IllegalNameException e){
			e.printStackTrace();		
		}
		return nodepath;
	}
	
	public Node removeChildNode(Node node,boolean recursive){
		if(node==null) throw new IllegalArgumentException("null");
		if(!recursive){
			try{
				node = internalRemoveChildNode(node);
			}catch(ItemNotFoundException e){
				return null;
			}
			return node;
		}
		NodeId id=node.getId();
		if(id==null)throw new IllegalArgumentException("null id");
		Node cnode=null;
		//try{
		cnode=this.getChildNode(id);
		//}catch(ItemNotFoundException e){
		//}
		
		if(cnode!=null){
			try{
				node = internalRemoveChildNode(node);
			}catch(ItemNotFoundException e){
				return null;
			}
			return node;
		}else{
			Collection<Node> children=this.getAllChildNode();
			for(Node child:children){
				Node removedNode=null;
				removedNode=child.removeChildNode(node,true);
				if(removedNode!=null)return removedNode;
			}
		}
		return null;
		
	}
	
	private Node internalRemoveChildNode(Node node)throws ItemNotFoundException{
		if(node==null) throw new IllegalArgumentException("null");
		NameInterface nodeName = node.getName();
		if(nodeName==null) throw new IllegalArgumentException("null");
		NodeId id=node.getId();
    	if(id!=null){
        	Node cnode=(Node)getChildNodes().get(id);
        	if(cnode==null) throw new ItemNotFoundException("nodeId:"+id);       		
        	
    	}else{
    		throw new ItemNotFoundException("null node id");
    	}
    	if(!this.fireBeforeRemoveChild())return node;
    	if(!node.fireBeforeBeRemoved())return node;
    	
    	ArrayList<NodeId> list = getChildEntries().get(nodeName);
    	if (list != null) {
    		list.remove(id);
    	}
    	Node dnode=(Node)getChildNodes().remove(id);
    	if(dnode!=null)node.setParentNull();
    	node.updateChildPath(true,false);
    	
    	node.fireAfterBeRemoved();
    	this.fireAfterRemoveChild();
    	return node;
	}
	
	public Node removeChildNode(NameInterface name,int index){
		if(name==null) throw new IllegalArgumentException("null");		
		ArrayList<NodeId> list=getChildEntries().get(name);
		if (list == null) return null;
		NodeId id=(NodeId)list.remove(index-PathElement.INDEX_DEFAULT);
		if(id==null) return null;
		Node node=(Node)getChildNodes().remove(id);
		if(node!=null)node.setParentNull();
		return node;	
	}
	
	public List<Node> removeChildNodesByName(NameInterface name){
		if(name==null) throw new IllegalArgumentException("null");		
		ArrayList<Node> removedList = new ArrayList<Node>();
		ArrayList<NodeId> list=getChildEntries().remove(name);
		if (list == null) return removedList;
		for(int i=0;i<list.size();i++){
			NodeId id=(NodeId)list.get(i);
			Node node=(Node)getChildNodes().remove(id);
			node.setParentNull();
			removedList.add(node);
		}		
		return removedList;		
	}
	
	public void removeAllChildNodes(){
		getChildEntries().clear();
		getChildNodes().clear();
	}
	

	

	

	/*
	public boolean orderBefore(String childname, int destIndex,int srcIndex)throws ItemNotFoundException{
		NameInterface n=null;
		try{
			n=NameImp.createName(childname);
		}catch(GLException e){
			throw new ItemNotFoundException();
		}
		return orderBefore(n,destIndex,srcIndex);
	}
	
	public synchronized boolean orderBefore(NameInterface childname, int destIndex,int srcIndex)throws ItemNotFoundException{
		if(childname==null||destIndex<PathElement.INDEX_DEFAULT||srcIndex<PathElement.INDEX_DEFAULT) throw new ItemNotFoundException();
		if(destIndex>srcIndex) return true;
        ArrayList<NodeId> list = getChildEntries().get(childname);
        if (list == null) throw new ItemNotFoundException();
        NodeId id=null;
    	if(list.size()<2) return false;
        id=(NodeId)list.remove(srcIndex-PathElement.INDEX_DEFAULT);
        System.out.println(id);
        list.add(destIndex-PathElement.INDEX_DEFAULT, id);
        System.out.println(destIndex);
        System.out.println(list.get(destIndex));
        //exchangeOrderWithDiffName();
        return true;
	}
	*/
	public synchronized boolean exchangeOrder(Node cnode1,Node cnode2){
		if(cnode1==null||cnode2==null) throw new IllegalArgumentException("null node");
		if(!hasChild(cnode1))return false;
		if(!hasChild(cnode2))return false;		
		NameInterface name1 = cnode1.getName();
		NameInterface name2 = cnode2.getName();		
		if(name1.equals(name2)){
			NodeId id1=cnode1.getId();
			NodeId id2=cnode2.getId();
			if(!exchangeOrderWithDiffName(id1,id2))return false;
	        ArrayList<NodeId> list = getChildEntries().get(name1);
	        if (list == null) throw new IllegalArgumentException();
	    	if(list.size()<2) return false;
	    	boolean exchanged = false;
	    	ArrayList<NodeId> newlist = new ArrayList<NodeId>();
	    	for(NodeId id:list){
	    		if(id.equals(id1)||id.equals(id2)){
					if(id.equals(id1)){
						newlist.add(id2);
					}else{
						newlist.add(id1);
					}
					exchanged=true;
					continue;	    			
	    		}
	    		newlist.add(id);
	    	}
			if(exchanged){
				this.setChildEntry(name1,newlist);
			}
			return exchanged;
		}else{
			return exchangeOrderWithDiffName(cnode1.getId(),cnode2.getId());
		}	
	}
	
	private boolean exchangeOrderWithDiffName(NodeId cnodeId1,NodeId cnodeId2){
		if(cnodeId1==null||cnodeId2==null) throw new IllegalArgumentException("null node");
		
		Node cnode1 = this.getChildNode(cnodeId1);
		Node cnode2 = this.getChildNode(cnodeId2);
		
		Collection<Node> list = getAllChildNode();
		//Node exchangeNode = null;
		boolean exchanged = false;
		LinkedMap childNodes  = new LinkedMap();
		for(Node cnode:list){
			NodeId id = cnode.getId();
			if(id.equals(cnodeId1)||id.equals(cnodeId2)){
				if(id.equals(cnodeId1)){
					childNodes.put(cnodeId2, cnode2);
				}else{
					childNodes.put(cnodeId1, cnode1);
				}
				exchanged=true;
				continue;
			}
			childNodes.put(id, cnode);
		}
		if(exchanged){
			this.setChildNodes(childNodes);
		}
		return exchanged;
	}
	
	public synchronized boolean orderBefore(Node cnode1,int destPos){
		if(cnode1==null) throw new IllegalArgumentException("null node");	
		Collection<Node> list = getAllChildNode();
		if(list==null||list.size()==0)return false;
		if(destPos<0&&destPos >= list.size()) throw new IllegalArgumentException("position illegal <0 or >=size()");
		
		List<Node> namenodelist = this.getChildNodeList(cnode1.getName());
		if(namenodelist == null)return false;
		NodeId id = cnode1.getId();
		int originPos = getChildNodeIndex(cnode1);
		if(originPos == destPos)return true;
		if(namenodelist.size() == 1){
			orderBeforeWithDiffName(id,destPos);
			return true;
		}
		//namenodelist.remove(cnode1);
		List<Node> adjustList = new ArrayList<Node>();		
		if(destPos<originPos) adjustList.add(cnode1);
		for(Node nNode : namenodelist){
			if(nNode==cnode1)continue;
			int pos = getChildNodeIndex(nNode);
			//System.out.println("pos:"+hasChild(nNode));
			//System.out.println("pos:"+pos);
			if((pos <= originPos && pos >= destPos) || (pos >= originPos && pos <= destPos)){
				adjustList.add(nNode);
			}
		}
		//System.out.println("adjustList:"+adjustList.size());
		if(destPos>originPos){
			System.out.println("adjustList:"+adjustList.size());
			adjustList.add(cnode1);
		}
		System.out.println("adjustList:"+adjustList.size());
		orderBeforeWithDiffName(id,destPos);
		ArrayList<NodeId> newNameList = new ArrayList<NodeId>();
		//namenodelist.add(cnode1);
		int k=0;
		for(int i=0;i<namenodelist.size();i++){
			Node nNode=namenodelist.get(i);
			if(!adjustList.contains(nNode)){
				newNameList.add(nNode.getId());
			}else{
				nNode=adjustList.get(k);
				newNameList.add(nNode.getId());
				if(nNode instanceof NodeImp){
					NodeImp imp = (NodeImp)nNode;
					//System.out.println("setPathRecursive:"+newNameList.size());
					setPathRecursive(imp,this,newNameList.size()-1);
				}
				k++;
			}
			/*
			if(adjustList.size()==1){
				newNameList.add(cnode1.getId());
				if(cnode1 instanceof NodeImp){
					NodeImp imp = (NodeImp)cnode1;
					setPathRecursive(imp,this,newNameList.size()-1);
				}				
			}*/
		}

		
		this.setChildEntry(cnode1.getName(), newNameList);
		return true;
	}
	
	private static void setPathRecursive(Node node,Node parent,int index){
		if(parent==null)return;
		if(index>=0){
			PathInterface nodepath=null;
			try{
				//if(!this.isRoot()){
				PathInterface parentPath=parent.getPath();
				//plus 1 coz want to be added as a  new child to parent
				nodepath=PathImp.createPath(parentPath,node.getName(),index+PathElement.INDEX_DEFAULT,true);				
				System.out.println(node.getId());
				System.out.println(nodepath);
				if(nodepath!=null) node.setPath(nodepath);
				//if(this.isNode()){
				//	this.updateChildPath(true);
				//}
			}catch(IllegalNameException e){
				e.printStackTrace();
				//throw new RuntimeException(e.getMessage());			
			}
		}else{
			if(node instanceof NodeImp){
				NodeImp imp = (NodeImp)node;
				imp.setPath(parent);
			}
			
		}

		Collection<Node> list=node.getAllChildNode();
		for(Node child:list){
			if(child instanceof NodeImp){
				NodeImp imp = (NodeImp)child;
				setPathRecursive(imp,node,-1);
			}
		}
	}
	
	private int orderBeforeWithDiffName(NodeId cnodeId1,int destPos){
		if(cnodeId1==null) throw new IllegalArgumentException("null node");		
		Node cnode1 = this.getChildNode(cnodeId1);
		Collection<Node> list = getAllChildNode();
		if(destPos<0&&destPos>=list.size()) throw new IllegalArgumentException("position illegal <0 or >=size()");
		list.remove(cnode1);
		Node[] nodeArray = list.toArray(new Node[0]);
		LinkedMap childNodes  = new LinkedMap();
		int iterationTimes = nodeArray.length;
		if(destPos==nodeArray.length){
			iterationTimes=nodeArray.length+1;
		}
		int originPos=-1;
		for(int i=0;i<iterationTimes;i++){
			if(i<nodeArray.length){
				Node cnode=nodeArray[i];
				NodeId id = cnode.getId();
				if(i==destPos){
					childNodes.put(cnodeId1, cnode1);
				}
				if(!id.equals(cnodeId1)){
					childNodes.put(id, cnode);
				}else{
					originPos=i;
				}				
			}else{
				if(i==destPos){
					childNodes.put(cnodeId1, cnode1);
				}				
			}

		}
		this.setChildNodes(childNodes);
		return originPos;
	}
	
	private int getChildNodeIndex(Node node){
		if(node==null) throw new IllegalArgumentException("null node");
		if(!hasChild(node))return -1;
		LinkedMap linkedMap = getChildNodes();
		return linkedMap.indexOf(node.getId());
	}
	

    
    public static boolean hasSameProperties(Node node1,Node node2){
    	Map<String,Property> properties1 = node1.getPropertyMap();
    	Map<String,Property> properties2 = node2.getPropertyMap();
    	if(properties1.size()!=properties2.size())return false;
    	for(String key:properties1.keySet()){
    		Property property2 = properties2.get(key);
    		if(property2==null)return false;
    		if(!properties1.equals(property2))return false;
    	}
    	return true;
    }
    
    public Property getProperty(String name){
    	if(name==null) return null;
    	//if(name.contains(Name.NAMESPACE_NAME_DELIMITER_LEFT)){
    		try {
				name=NameImp.createName(name).getLocalName();
			} catch (IllegalNameException e) {
				e.printStackTrace();
				return null;
			}
    	//}
    	return getPropertyMap().get(name);
    }
	
    public Property addProperty(String name, Property property)throws ItemExistsException{
    	if(property==null) throw new IllegalArgumentException("null property");
    	Property prop=getProperty(name);
    	
    	if(prop!=null) throw new ItemExistsException(name);
    	
    	//if(!property.checkBeforeBeAdded()) throw new InvalidItemStateException();
    	getPropertyMap().put(name, property);

    	return property;
    }
    
    public Property setOrAddProperty(String name, Property prop){
    	if(name==null) throw new IllegalArgumentException("null name");
    	Property property=getProperty(name);
    	if(property==null){
    		try {
				return addProperty(name,prop);
			} catch (ItemExistsException e) {
			}
    	}   	
		property.setValue(prop.getValue());
    	return property;
    }
    public Property setOrAddProperty(String name, String proValue,String proType) throws IllegalValueException{
    	if(name==null) throw new IllegalArgumentException("null name");
    	if(proValue==null) throw new IllegalArgumentException("null proValue");
    	Property property=null;
    	property=PropertyImp.create(name,proValue,proType);
		return setOrAddProperty(name,property);
    }
    
    public Property setOrAddProperty(String name, String proValue,int proType) throws IllegalValueException{
    	if(name==null) throw new IllegalArgumentException("null name");
    	if(proValue==null) throw new IllegalArgumentException("null proValue");
    	Property property=null;
    	property=PropertyImp.create(name, proValue,proType);
		return setOrAddProperty(name,property);
    }
    
    public Property setOrAddProperty(String name, Value value) throws IllegalValueException{
    	if(name==null) throw new IllegalArgumentException("null name");
    	if(value==null) throw new IllegalArgumentException("null value");
    	Property property=PropertyImp.create(name, value);
    	return setOrAddProperty(name,property);
    }
    
    public Property removeProperty(String name)throws ItemNotFoundException{
    	if(name==null) throw new IllegalArgumentException("null name");
    	Property property=getProperty(name);
    	if(property==null) throw new ItemNotFoundException(this.getPath().toString()+PathInterface.DELIMITER+name.toString());
    	return getPropertyMap().remove(property);
    }
    
    

    

	
	public List<Node> extract(NodeBeanFilter filter,boolean recursive){
		if(filter==null)throw new IllegalArgumentException();
		List<Node> nodelist=new ArrayList<Node>();
		Collection<Node> children=this.getAllChildNode();
		if(children==null)return nodelist;
		if(children.size()==0)return nodelist;
		for(Node node: children){
			if(filter.accept(node)){
				nodelist.add(node);
			}
			if (recursive) {
				if(node instanceof NodeImp){
					NodeImp rnode=(NodeImp)node;
					nodelist.addAll(rnode.extract(filter, recursive));	
				}
			}
		}
		return nodelist;
	}
	
	public List<Node> extractExtended(NodeBeanFilter filter){
		if(filter==null)throw new IllegalArgumentException();
		List<Node> nodelist=new ArrayList<Node>();
		Collection<Node> children=this.getAllLogicChildNode();
		if(children==null)return nodelist;
		if(children.size()==0)return nodelist;
		for(Node node: children){		
			if(filter.accept(node)){
				nodelist.add(node);
			}
			//if (recursive) {
				if(node instanceof NodeImp){
					NodeImp rnode=(NodeImp)node;
					nodelist.addAll(rnode.extractExtended(filter));	
				}
			//}
		}
		return nodelist;
	}
	
	public List<Node> extractStruct(NodeBeanFilter filter,boolean recursive){
		if(filter==null)throw new IllegalArgumentException();
		List<Node> nodelist=new ArrayList<Node>();
		Collection<Node> children=this.getAllLogicChildNode();
		if(children==null)return nodelist;
		if(children.size()==0)return nodelist;
		for(Node node: children){		
			if(filter.accept(node)){
				nodelist.add(node);
			}
			if (recursive) {
				if(node instanceof NodeImp){
					NodeImp rnode=(NodeImp)node;
					nodelist.addAll(rnode.extractStruct(filter,recursive));	
				}
			}
		}
		return nodelist;
	}
	

	
	public boolean equalsRecursive(Object obj){
		if(!this.equals(obj))return false;
		if (obj instanceof NodeImp) {
			NodeImp otherNode = (NodeImp) obj;
			Collection<Node> children = this.getAllChildNode();
			Collection<Node> children_other = otherNode.getAllChildNode();
	    	
			Node[] children_array = children.toArray(new Node[0]);
			Node[] children_array_other = children_other.toArray(new Node[0]);
			
			if(children.size()!=children_other.size()) return false;
	    	for(int i=0;i<children_array.length;i++){
	    		
	    		Node node=children_array[i];
	    		Node oNode =children_array_other[i];
	    		if(!node.equals(oNode))return false;
	    	}
	    	return true;			
		}
		return false;
	}
	

	
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof NodeImp) {
        	NodeImp otherNode = (NodeImp) obj;
        	//NodeId otherId=otherNode.getId();
        	if(!otherNode.getId().equals(this.getId()))return false;
        	if(!hasSameProperties(this,otherNode)) return false;
        	return true;	
        }
        return false;
    }

	public boolean updateProperty(String fieldName) {
		Value value = ValueFactoryImp.genValue(this, fieldName);
		// System.out.println(value == null);
		if (value == null) return false;
		try {
			// System.out.println(value.getString());
			this.setOrAddProperty(fieldName, value);
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
    
	public void newId(boolean recursive){
		NodeId nodeid=NodeId.createInstance();
		String newId=nodeid.toString();
		//System.out.println("newId:"+newId);
		this.setId(newId);
		//System.out.println("newId2:"+this.getId().toString());
		//this.treeId=null;
		try {
			// System.out.println(value.getString());
			this.setOrAddProperty(Node.ID_PROPERTY_LITERAL, newId,PropertyType.STRING);
		} catch (Exception e) {
			e.printStackTrace();
			//return false;
		}
		//updateProperty(Node.ID_PROPERTY_LITERAL);
		if(recursive){
			Collection<Node> children=this.getAllChildNode();
			List<Node> clist = new ArrayList<Node>();
			for(Node cnode:children){
				clist.add(cnode);
			}
			for(Node cnode:clist){
				cnode.newId(recursive);
			}
		}
	}
    
	public List<Node> getParentNodeList(){
		List<Node> parentlist = new ArrayList<Node>();
		Node parent = this.getParent();
		if(parent==null) return parentlist;
		
		while(parent!=null){
			parentlist.add(parent);
			parent = parent.getParent();
		}
		
		return parentlist;
	}
	
	public List<Node> getParentNodeList(boolean reverse){
		List<Node> list = getParentNodeList();
		if(!reverse){
			return list;
		}
		Collections.reverse(list);
		
		return list;
	}
	
	public Map<String,Property> getPropertyMap(){
		return _properties;
	}
	
	public void setPropertyMap(Map properties){
		_properties = properties;
	}
	
	public String getIdString(){
		return _nodeId.toString();
	}
    
	public boolean isNode(){
		return true;
	}
	
	public boolean isRoot(){
		return this.getParent()==null;
	}
	
	public boolean isLogicRoot(){
		return this.getPath().isRoot();
	}
	
	private LinkedMap get_ChildNodes(){
		return _childNodes;
	}
	
	public Map<NameInterface, ArrayList<NodeId>> getChildEntries() {
		return _childEntries;
	}
	
	public Map<NameInterface, ArrayList<NodeId>> getLogicChildEntries() {
		return getChildEntries();
	}
	
	public LinkedMap getChildNodes(){
		return _childNodes;
	}
	
	public LinkedMap getLogicChildNodes(){
		return getChildNodes();
	}
	
	public void setChildNodes(LinkedMap map){
		_childNodes=map;
	}
		
	public void setChildEntries(HashMap<NameInterface, ArrayList<NodeId>> entries) {
		_childEntries = entries;
	}
	
	public Node getLogicPeer(){
		return this;
	}
	
	protected void setId(String id){
		if(id==null||id.trim().equals("")){
			return;
		}
		try{
			NodeId newId=NodeId.valueOf(id);
			setId(newId);
		}catch(Exception e){
			e.printStackTrace();
			return;
		}
	}
	
	protected void setId(NodeId newId){
		if(newId==null){
			return;
		}
		Node nparent = this.getParent();
		if(nparent==null){
			_nodeId=newId;
			return;
		}
		if(nparent instanceof NodeImp){
			NodeImp parent = (NodeImp)nparent;
			ArrayList<NodeId> siblings=parent.getChildEntries().get(this.getName());
			for(int i=0;i<siblings.size();i++){
				NodeId sib = siblings.get(i);
				if(sib.equals(this.getId())){
					siblings.set(i, newId);
					break;
				}
			}
			LinkedMap cmap = parent.get_ChildNodes();
			cmap.remove(this.getId());
			cmap.put(newId, this);
		}

		/*
		List<Node> siblings=getChildNodeList(this.getName());
		for(int i=0;i<siblings.size();i++){
			Node sib = siblings.get(i);
			if(sib.getId().equals(this.getId())){
				siblings.set(i, arg1)
			}
		}*/
	}
	private static Property genIdProperty(NodeId  nodeId){
		if(nodeId==null)return null;
		NameInterface proName=null;
		try {
			proName = NameImp.createName(Node.ID_PROPERTY_LITERAL);
			Property property = PropertyImp.create(proName, nodeId.toString(), PropertyType.STRING);
			return property;
			//this.addProperty(Node.ID_PROPERTY_LITERAL, property);
		} catch (GLException e) {
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/*
	public void updateChildPath(boolean recursive,boolean extend){
		Collection<Node> children = null;
		if(!extend){
			children = getAllChildNode();
		}else{
			children = getAllLogicChildNode();
		}
		for(Node cnode : children){
			cnode.setPath(this);
			if(recursive){
				cnode.updateChildPath(recursive,extend);
			}
		}
	}*/
	public void updateChildPath(boolean recursive,boolean extend){
		Collection<Node> children = null;
		if(!extend){
			children = getAllChildNode();
		}else{
			children = getAllLogicChildNode();
		}
		PathInterface path= this.getPath();
		for(Node cnode:children){
			PathInterface cpath = cnode.getPath();
			PathElement cPathEle = cpath.getNameElement();
			try{
				PathInterface nodepath=PathImp.createPath(path,cPathEle.getName(),cPathEle.getIndex(),true);	
				if(nodepath!=null){
					cnode.setPath(nodepath);
				}
			}catch(Exception e){
				e.printStackTrace();
			}
			if(recursive){
				cnode.updateChildPath(recursive,extend);
			}
		}
	}
	
	
    public boolean fireBeforeAddChild(){
    	return true;
    }
    public boolean fireAfterAddChild(){
    	return true;
    }
	
    public boolean fireBeforeBeAdded(){
    	return true;
    }
    public boolean fireAfterBeAdded(){
    	return true;
    }
	
    public boolean fireBeforeRemoveChild(){
    	return true;
    }
    public boolean fireAfterRemoveChild(){
    	return true;
    }
	
    public boolean fireBeforeBeRemoved(){
    	return true;
    }
    public boolean fireAfterBeRemoved(){
    	return true;
    }
	
	public NodeId getId(){
		return _nodeId;
	}
	public JSONArray getIdentityChildrenJsonArray(){
		return new JSONArray();
	}
	public JSONObject toIdentityJson(){
		return new JSONObject();
	}
	public String getType(){
		return NodeImp.NODE_TYPE;
	}
}
