package com.giantleap.common.nodebean.node.base.storage.path;

import java.io.Serializable;

/**
 * A <code>Name</code> is a combination of a namespace URI and a local part.
 * Instances of this class are used to internally represent the names of JCR
 * content items and other objects within a content repository.
 * <p/>
 * A <code>Name</code> is immutable once created.
 * <p/>
 * The String representation of a <code>Name</code> object must be in the
 * format "<code>{namespaceURI}localPart</code>".
 * <p/>
 * An implementation of the <code>Name</code> interface must implement the
 * {@link Object#equals(Object)} method such that two Name objects are equal if
 * both the namespace URI and the local part are equal.
 */
public interface NameInterface  extends Comparable, Cloneable, Serializable{

    public final static NameInterface ROOT_NAME    =  NameImp.createName(NameInterface.NS_DEFAULT_URI, PathInterface.ROOT_PATH_LITERAL);
    public final static NameInterface CURRENT_NAME =  NameImp.createName(NameInterface.NS_DEFAULT_URI, PathInterface.CURRENT_PATH_LITERAL);
    public final static NameInterface PARENT_NAME  =  NameImp.createName(NameInterface.NS_DEFAULT_URI, PathInterface.PARENT_PATH_LITERAL);
    
    
    public static final String NAMESPACE_NAME_DELIMITER_LEFT     = "{";
    public static final String NAMESPACE_NAME_DELIMITER_RIGHT    = "}";
    
    public static final String NAMESPACE_NAME_XML_DELIMITER      =":";
    public static final char   NAMESPACE_NAME_XML_DELIMITER_CHAR =':';
    
    public static final char NAMESPACE_NAME_DELIMITER_LEFT_CHAR  = '{';
    public static final char NAMESPACE_NAME_DELIMITER_RIGHT_CHAR = '}';
    


    
    // default namespace (empty uri)    
    public static final String NS_EMPTY_PREFIX = "";
    public static final String NS_DEFAULT_URI = "";

    // reserved namespace for repository internal node types
    public static final String NS_REP_PREFIX = "rep";
    public static final String NS_REP_URI = "internal";

    // reserved namespace for items defined by built-in node types
    public static final String NS_JCR_PREFIX = "jcr";
    public static final String NS_JCR_URI = "http://www.jcp.org/jcr/1.0";

    // reserved namespace for built-in primary node types
    public static final String NS_NT_PREFIX = "nt";
    public static final String NS_NT_URI = "http://www.jcp.org/jcr/nt/1.0";

    // reserved namespace for built-in mixin node types
    public static final String NS_MIX_PREFIX = "mix";
    public static final String NS_MIX_URI = "http://www.jcp.org/jcr/mix/1.0";

    // reserved namespace used in the system view XML serialization format
    public static final String NS_SV_PREFIX = "sv";
    public static final String NS_SV_URI = "http://www.jcp.org/jcr/sv/1.0";

    // reserved namespaces that must not be redefined and should not be used
    public static final String NS_XML_PREFIX = "xml";
    public static final String NS_XML_URI = "http://www.w3.org/XML/1998/namespace";
    public static final String NS_XMLNS_PREFIX = "xmlns";
    public static final String NS_XMLNS_URI = "http://www.w3.org/2000/xmlns/";

    

    /**
     * Empty array of <code>Name</code>
     */
    //public static final Name[] EMPTY_ARRAY = new Name[0];

    /**
     * Returns the local part of this <code>Name</code> object.
     *
     * @return local name
     */
    public String getLocalName();

    /**
     * Returns the namespace URI of this <code>Name</code> object.
     *
     * @return namespace URI
     */
    public String getNamespaceURI();
    
    public String getXMLString();
}