package com.giantleap.common.nodebean.node.base.storage.path;

import java.io.Serializable;


public interface PathElement extends Serializable {
	
	
    public static final PathElement CURRENT_ELEMENT = PathElementImp.createPathElement(NameInterface.CURRENT_NAME,PathElement.INDEX_UNDEFINED,2);
    public static final PathElement PARENT_ELEMENT  = PathElementImp.createPathElement(NameInterface.PARENT_NAME,PathElement.INDEX_UNDEFINED,4);
    public static final PathElement ROOT_ELEMENT    = PathElementImp.createPathElement(NameInterface.ROOT_NAME,PathElement.INDEX_UNDEFINED,1);
	
		
	
    public static final String INDEX_DELIMITER_LEFT     = "[";
    public static final String INDEX_DELIMITER_RIGHT    = "]";
    
    public static final char INDEX_DELIMITER_LEFT_CHAR  = '[';
    public static final char INDEX_DELIMITER_RIGHT_CHAR = ']';
   
    /**
     * Constant representing an undefined index value
     */
    public static final int INDEX_UNDEFINED = 0;

    /**
     * Constant representing the default (initial) index value.
     */
    public static final int INDEX_DEFAULT = 0;
    
    public final static int NORMAL_TYPE  = 0;
    public final static int ROOT_TYPE    = 1;
    public final static int CURRENT_TYPE = 2;
    public final static int PARENT_TYPE  = 4;
    
    /**
     * Returns the qualified name of this path element.
     *
     * @return qualified name
     */
    public NameInterface getName();

    /**
     * Returns the index of the element as it has been assigned upon creation.
     *
     * @return index of the element as it has been assigned upon creation.
     */
    public int getIndex();

    /**
     * Returns the normalized index of this path element, i.e. the index
     * is always equals or greater that {@link Path#INDEX_DEFAULT}.
     *
     * @return the normalized index.
     */
    public int getNormalizedIndex();

    /**
     * Returns <code>true</code> if this element denotes the <i>root</i> element,
     * otherwise returns <code>false</code>.
     *
     * @return <code>true</code> if this element denotes the <i>root</i>
     *         element; otherwise <code>false</code>
     */
    public boolean denotesRoot();

    /**
     * Returns <code>true</code> if this element denotes the <i>parent</i>
     * ('..') element, otherwise returns <code>false</code>.
     *
     * @return <code>true</code> if this element denotes the <i>parent</i>
     *         element; otherwise <code>false</code>
     */
    public boolean denotesParent();

    /**
     * Returns <code>true</code> if this element denotes the <i>current</i>
     * ('.') element, otherwise returns <code>false</code>.
     *
     * @return <code>true</code> if this element denotes the <i>current</i>
     *         element; otherwise <code>false</code>
     */
    public boolean denotesCurrent();

    /**
     * Returns <code>true</code> if this element represents a regular name
     * (i.e. neither root, '.' nor '..'), otherwise returns <code>false</code>.
     *
     * @return <code>true</code> if this element represents a regular name;
     *         otherwise <code>false</code>
     */
    public boolean denotesRegularName();

    /**
     * Return the String presentation of a{@link Element}. It must be
     * in the format "<code>{namespaceURI}localPart</code>" or
     * "<code>{namespaceURI}localPart[index]</code>" in case of an index
     * greater than {@link Path#INDEX_DEFAULT}.
     * 
     * @return
     */
    public String getString();
    public String getXMLString();
}