package com.giantleap.common.nodebean.node.base.storage.path;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import com.giantleap.common.exception.GLException;
import com.giantleap.common.nodebean.node.base.storage.path.exception.IllegalNameException;
import com.giantleap.common.nodebean.node.base.storage.path.exception.IllegalPathException;
import com.giantleap.common.nodebean.node.base.storage.path.exception.PathNotFoundException;


public final class PathImp implements PathInterface {

    /**
     * the elements of this path
     */
    private final PathElement[] elements;

    /**
     * flag indicating if this path is normalized
     */
    private final boolean normalized;

    /**
     * flag indicating if this path is absolute
     */
    private final boolean absolute;

    /**
     * the cached hashcode of this path
     */
    private transient int hash = 0;

    /**
     * the cached 'toString' of this path
     */
    //private transient String string;

    public PathImp(PathElement[] elements, boolean isNormalized) {
        if (elements == null || elements.length == 0) {
            throw new IllegalArgumentException("Empty paths are not allowed");
        }
        this.elements = elements;
        this.absolute = elements[0].denotesRoot();
        this.normalized = isNormalized;
    }

    public static PathInterface createPath(PathInterface parent, NameInterface name, int index, boolean normalize) throws IllegalNameException{
        if(parent==null) throw new RuntimeException();
    	List<PathElement> elements = new ArrayList<PathElement>();
        elements.addAll(Arrays.asList(parent.getElements()));
        elements.add(PathElementImp.createPathElement(name, index));
        try{
            PathInterface path=createPath(elements);
            if (normalize) {
                return path.getNormalizedPath();
            } else {
                return path;
            }       	
        }catch(GLException e){
        	throw new IllegalNameException(e.getMessage());
        }

    }
    
    public static PathInterface createPath(String name)throws IllegalNameException {
    	return doParse(name);
    }
    
    public static PathInterface createPath(NameInterface name)throws IllegalNameException {
    	PathElement elem = PathElementImp.createPathElement(name);
    	PathElement[] eles=new PathElement[]{elem};
        return createPath(eles);
    }
    
    private static PathInterface doParse(String pathString)throws IllegalNameException {
        if (pathString == null || "".equals(pathString.trim())) {
            throw new IllegalArgumentException("Invalid Path literal");
        }
        pathString=pathString.trim();
        int lastPos = 0;
        
        ArrayList<PathElement> list = new ArrayList<PathElement>();
        
        if(pathString.charAt(0)==PathInterface.ROOT_PATH_CHAR_LITERAL){
        	list.add(PathElement.ROOT_ELEMENT);
        	pathString=pathString.substring(1);
        }
        //System.out.println(pathString.length());
        // split into path elements
        int pos = pathString.indexOf(PathInterface.DELIMITER);
        //System.out.println(pos);
        while (lastPos >= 0) {
        	PathElement elem=null;
            if (pos >= 0) {
                elem = PathElementImp.createPathElement(pathString.substring(lastPos, pos));
                lastPos = pos + 1;
                pos = pathString.indexOf(PathInterface.DELIMITER, lastPos);
                list.add(elem);
            } else {
            	//System.out.println(lastPos);
            	//System.out.println(pathString.length());
            	if(pathString.length()>0&&lastPos<pathString.length()-1){
            		//String subPath = pathString.substring(lastPos-1);
            		//System.out.println(subPath);
            		//if(subPath.length()==1 && subPath.charAt(0)==PathInterface.ROOT_PATH_CHAR_LITERAL) break;
                    elem = PathElementImp.createPathElement(pathString.substring(lastPos));                       
                    
            	}
            	
            	if(elem!=null)list.add(elem);
            	break;
            }
            
        }
        //System.out.println("list.size():"+list.size());
        return createPath(list);
    }
    public static PathInterface createPath(List<PathElement> list){
    	PathElement[] eles=(PathElement[])(list.toArray(new PathElement[list.size()]));
    	//System.out.println(list.size());
    	return createPath(eles);
    }
    public static PathInterface createPath(PathElement[] eles){
        if (eles == null || eles.length == 0) {
            throw new IllegalArgumentException("Cannot build path from null or 0 elements.");
        }
       // boolean leadingParent=false;
        boolean isNormalized =true;
        //if(eles[0].denotesParent()) leadingParent=true;
        for (int i = 0; i < eles.length; i++) {        	
        	PathElement elem  = eles[i];
            isNormalized  = (isNormalized && elem.denotesParent()&&elem.denotesCurrent());          
        }
        return new PathImp(eles, isNormalized);
    }
    
    
    /**
     * @see PathInterface#denotesRoot()
     */
    public boolean denotesRoot() {
        return absolute && elements.length == 1;
    }

    /**
     * @see PathInterface#isAbsolute()
     */
    public boolean isAbsolute() {
        return absolute;
    }
    

    /**
     * @see PathInterface#isCanonical()
     */
    public boolean isCanonical() {
        return absolute && normalized;
    }

    /**
     * @see PathInterface#isNormalized()
     */
    public boolean isNormalized() {
        return normalized;
    }

    /**
     * @see PathInterface#getNormalizedPath()
     
    public PathInterface getNormalizedPath() throws RepositoryException {
    	return null;
    }*/
   
    public PathInterface getNormalizedPath() throws IllegalPathException {
        if (isNormalized()) {
            return this;
        }
        return getNormalizedPathHelper(elements);
    }
    
    
    public static PathInterface getNormalizedPathHelper(List list) throws IllegalPathException {
    	PathElement[] eles=(PathElement[])(list.toArray(new PathElement[list.size()]));
    	return getNormalizedPathHelper(eles);
    }
    
    public static PathInterface getNormalizedPathHelper(PathElement[] elements) throws IllegalPathException {
    	LinkedList queue = new LinkedList();
    	PathElement last = PathElementImp.PARENT_ELEMENT;
        for (int i = 0; i < elements.length; i++) {
        	PathElement elem = elements[i];
        	//System.out.println(i+" "+elem+" "+elem.denotesRoot());
            if (elem.denotesParent() && !last.denotesParent()) {
                if (last.denotesRoot()) {
                    // the first element is the root element;
                    // ".." would refer to the parent of root
                    throw new IllegalPathException("PathInterface can not be canonicalized: unresolvable '..' element");
                }
                queue.removeLast();
                if (queue.isEmpty()) {
                    last = PathElementImp.PARENT_ELEMENT;
                } else {
                    last = (PathElement) queue.getLast();
                }
            } else if (!elem.denotesCurrent()) {
            	queue.add(elem);
                last = elem;                
            }else if(elem.denotesCurrent()){
            	
            }
        }
        if (queue.isEmpty()) {
            throw new IllegalPathException("PathInterface can not be normalized: would result in an empty path.");
        }
        boolean isNormalized = true;
        return new PathImp((PathElement[]) queue.toArray(new PathElement[queue.size()]), isNormalized);
    }
    /**
     * @see PathInterface#getCanonicalPath()
     */
    public PathInterface getCanonicalPath() throws IllegalPathException {
        if (isCanonical()) {
            return this;
        }
        if (!isAbsolute()) {
            throw new IllegalPathException("Only an absolute path can be canonicalized.");
        }
        return getNormalizedPath();
    }

    /**
     * @see PathInterface#computeRelativePath(PathInterface)
     */
    public PathInterface computeRelativePath(PathInterface other) {
        if (other == null) {
            throw new IllegalArgumentException("null argument");
        }

        // make sure both paths are absolute
        if (!isAbsolute() || !other.isAbsolute()) {
            throw new RuntimeException("Cannot compute relative path from relative paths");
        }

        // make sure we're comparing canonical paths
        PathInterface p0 = null;
        PathInterface p1 = null;
        try{
        	p0 = getCanonicalPath();
        	p1 = other.getCanonicalPath();
        }catch(Exception e){
        	return null;
        }


        if (p0.equals(p1)) {
            // both paths are equal, the relative path is therefore '.'
            //Builder pb = new Builder(new PathInterface.Element[] {CURRENT_ELEMENT});
            //return pb.getPath();
        	return CURRENT_PATH;
        }

        // determine length of common path fragment
        int lengthCommon = 0;
        PathElement[] elems0 = p0.getElements();
        PathElement[] elems1 = p1.getElements();
        for (int i = 0; i < elems0.length && i < elems1.length; i++) {
            if (!elems0[i].equals(elems1[i])) {
                break;
            }
            lengthCommon++;
        }
        List<PathElement> elemList = new ArrayList<PathElement>();
        if (lengthCommon < elems0.length) {
            /**
             * the common path fragment is an ancestor of this path;
             * this has to be accounted for by prepending '..' elements
             * to the relative path
             */
            int tmp = elems0.length - lengthCommon;
            while (tmp-- > 0) {
            	elemList.add(0, PathElementImp.PARENT_ELEMENT);
            }
        }
        // add remainder of other path
        for (int i = lengthCommon; i < elems1.length; i++) {
        	elemList.add(elems1[i]);
        }       
        return createPath(elemList);
    }

    /**
     * @see PathInterface#getAncestor(int)
     */
    public PathInterface getAncestor(int degree) throws PathNotFoundException {
        if (degree < 0) {
            throw new IllegalArgumentException("degree must be >= 0");
        } else if (degree == 0) {
            return this;
        }
        int length = elements.length - degree;
        if (length < 1) {
            throw new PathNotFoundException("no such ancestor path of degree " + degree);
        }
        PathElement[] elements = new PathElement[length];
        System.arraycopy(this.elements, 0, elements, 0, length);
        return new PathImp(elements, normalized);
    }

    /**
     * @see PathInterface#getAncestorCount()
     */
    public int getAncestorCount() {
        return getDepth() - 1;
    }

    /**
     * @see PathInterface#getLength()
     */
    public int getLength() {
        return elements.length;
    }

    /**
     * @see PathInterface#getDepth()
     */
    public int getDepth() {
        int depth = ROOT_DEPTH;
        for (int i = 0; i < elements.length; i++) {
            if (elements[i].denotesParent()) {
                depth--;
            } else if (!elements[i].denotesCurrent()) {
                depth++;
            }
        }
        return depth;
    }

    /**
     * @see PathInterface#isAncestorOf(PathInterface)
     */
    public boolean isAncestorOf(PathInterface other) throws  IllegalPathException {
        if (other == null) {
            throw new IllegalArgumentException("null argument");
        }
        // make sure both paths are either absolute or relative
        if (isAbsolute() != other.isAbsolute()) {
            throw new IllegalArgumentException("Cannot compare a relative path with an absolute path");
        }
        // make sure we're comparing normalized paths
        PathInterface p0 = getNormalizedPath();
        PathInterface p1 = other.getNormalizedPath();

        if (p0.equals(p1)) {
            return false;
        }
        // calculate depth of paths (might be negative)
        if (p0.getDepth() >= p1.getDepth()) {
            return false;
        }
        PathElement[] elems0 = p0.getElements();
        PathElement[] elems1 = p1.getElements();
        for (int i = 0; i < elems0.length; i++) {
            if (!elems0[i].equals(elems1[i])) {
                return false;
            }
        }
        return true;
    }

    /**
     * @see PathInterface#isDescendantOf(PathInterface)
     */
    public boolean isDescendantOf(PathInterface other) throws IllegalPathException {
        if (other == null) {
            throw new IllegalArgumentException("Null argument");
        }
        return other.isAncestorOf(this);
    }

    /**
     * @see PathInterface#subPath(int, int)
     */
    public PathInterface subPath(int from, int to) throws IllegalPathException {
        if (from < 0 || to >= elements.length || from >= to) {
            throw new IllegalArgumentException();
        }
        if (!isNormalized()) {
            throw new IllegalPathException("Cannot extract sub-PathInterface from a non-normalized PathInterface.");
        }
        PathElement[] dest = new PathElement[to-from];
        System.arraycopy(elements, from, dest, 0, dest.length);
        return PathImp.createPath(dest);
    }


    public boolean isRoot(){
    	PathInterface path=null;
    	try{
    		path=this.getNormalizedPath();
    	}catch(GLException e){
    		e.printStackTrace();
    	}
    	if(path==null) return false;
    	return (path.getElements().length==1&&path.getNameElement()==PathElementImp.ROOT_ELEMENT);
    }
    
    public static PathInterface getChildPath(PathInterface path){
    	if(path==null) return null;
    	PathElement[] eles=path.getElements();
    	if(eles==null||eles.length<=1) return null;
    	PathElement[] childEle=new PathElement[eles.length-1];
    	for(int i=0;i<eles.length-1;i++){
    		childEle[i]=eles[i+1];
    	}
    	return createPath(childEle);
    }
    
    
    /**
     * @see PathInterface#getNameElement()
     */
    public PathElement getNameElement() {
        return elements[elements.length - 1];
    }    
    public String  getNameString(){
    	return getNameElement().getName().toString();
    }
    public String  getElementString(){
    	return getNameElement().getString();
    }

    /**
     * @see PathInterface#getString()
     */
    public String getString() {
        return toString();
    }

    /**
     * @see PathInterface#getElements()
     */
    public PathElement[] getElements() {
        return elements;
    }

    //---------------------------------------------------------< Object >---
    /**
     * Returns the internal string representation of this <code>PathInterface</code>.
     * <p/>
     * Note that the returned string is not a valid JCR path, i.e. the
     * namespace URI's of the individual path elements are not replaced with
     * their mapped prefixes.
     *
     * @return the internal string representation of this <code>PathInterface</code>.
     */
    public String toString() {
        // PathInterface is immutable, we can store the string representation
       // if (string == null) {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < elements.length; i++) {
                if(!isAbsolute()){
                	if (i > 0) {
                        sb.append(PathInterface.DELIMITER);
                    }               	
                }else{
                	if (i > 1) {
                        sb.append(PathInterface.DELIMITER);
                    }                   	
                }
                PathElement element = elements[i];
                String elem = element.toString();
                sb.append(elem);
            }
            //string = sb.toString();
        //}
        return sb.toString();
    }

    public String getXMLString() {
        // PathInterface is immutable, we can store the string representation
        //if (string == null) {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < elements.length; i++) {
                if(!isAbsolute()){
                	if (i > 0) {
                        sb.append(PathInterface.DELIMITER);
                    }               	
                }else{
                	if (i > 1) {
                        sb.append(PathInterface.DELIMITER);
                    }                   	
                }

                PathElement element = elements[i];
                String elem = element.getXMLString();
                sb.append(elem);
            }
           // string = sb.toString();
       // }
        return sb.toString();
    }
    
    /**
     * Returns a hash code value for this path.
     *
     * @return a hash code value for this path.
     * @see Object#hashCode()
     */
    public int hashCode() {
        // PathInterface is immutable, we can store the computed hash code value
        int h = hash;
        if (h == 0) {
            h = 17;
            for (int i = 0; i < elements.length; i++) {
                h = 37 * h + elements[i].hashCode();
            }
            hash = h;
        }
        return h;
    }

    /**
     * Compares the specified object with this path for equality.
     *
     * @param obj the object to be compared for equality with this path.
     * @return <tt>true</tt> if the specified object is equal to this path.
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof PathInterface) {
            PathInterface other = (PathInterface) obj;
            return Arrays.equals(elements, other.getElements());
        }
        return false;
    }
    
    
    public PathElement getFirstElement(){
    	return elements[0];
    }
    
    

    public static void main(String[] args) throws Exception{ 	
    	//PathElement e=PathElementFactoryImp.createHelper(Name.ROOT_NAME);
    	//System.out.println(Name.ROOT_NAME);
    	//Path p=createHelper(Name.ROOT_NAME);
    	//Name name=NameFactoryImp.createHelper("","fsa");
    	//Path c=createHelper(p,name,true);
    	//System.out.println(c);
    	
    	PathInterface d=createPath("/fs/sfd:jh[1]");
    	System.out.println(d.getString());
    	System.out.println(d.getElementString());
    	System.out.println(d.getXMLString());
    	
    	PathInterface d2=PathInterface.ROOT_PATH;
    	System.out.println(d2.getElements().length);
    	
    	NameInterface name = NameInterface.ROOT_NAME;
    	System.out.println(name.getLocalName());
    	//Path o=((Path)d.getNormalizedPath());
    	//System.out.println(o);
    }
}