package com.giantleap.common.nodebean.node.base.storage.value;

public class HTTPLinkValue  extends BaseValue{
	public static final int TYPE = PropertyType.HTTPLINK;
	private final String _text;
    /**
     * Constructs a <code>StringValue</code> object representing a string.
     *
     * @param text the string this <code>StringValue</code> should represent
     */
    public HTTPLinkValue(String text) {
        super(TYPE);
        this._text = text;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * <p/>
     * The result is <code>true</code> if and only if the argument is not
     * <code>null</code> and is a <code>StringValue</code> object that
     * represents the same value as this object.
     *
     * @param obj the reference object with which to compare.
     * @return <code>true</code> if this object is the same as the obj
     *         argument; <code>false</code> otherwise.
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof HTTPLinkValue) {
        	HTTPLinkValue other = (HTTPLinkValue) obj;
            if (_text == other._text) {
                return true;
            } else if (_text != null && other._text != null) {
                return _text.equals(other._text);
            }
        }
        return false;
    }

    /**
     * Returns zero to satisfy the Object equals/hashCode contract.
     * This class is mutable and not meant to be used as a hash key.
     *
     * @return always zero
     * @see Object#hashCode()
     */
    public int hashCode() {
        return 0;
    }
    
    public static HTTPLinkValue valueOf(String s){

    	return new HTTPLinkValue(s);
    }

    //------------------------------------------------------------< BaseValue >
    /**
     * {@inheritDoc}
     */
    protected String getInternalString() throws IllegalValueException {
        if (_text != null) {
            return _text;
        } else {
            throw new IllegalValueException("empty value");
        }
    }	
}
	

