package com.giantleap.common.nodebean.node.base.storage.value;

import java.util.Calendar;
import java.util.Date;

public class IntegerValue extends BaseValue {
	public static final int TYPE = PropertyType.INTEGER;

	private final Integer lInteger;

	public IntegerValue(Integer lInteger) {
		super(TYPE);
		this.lInteger = lInteger;
	}

	public IntegerValue(int i) {
		super(TYPE);
		this.lInteger = new Integer(i);
	}

	public static IntegerValue valueOf(String s) throws IllegalValueException {
		try {
			return new IntegerValue(Integer.parseInt(s));
		} catch (NumberFormatException e) {
			throw new IllegalValueException("invalid format", e);
		}
	}

	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj instanceof IntegerValue) {
			IntegerValue other = (IntegerValue) obj;
			if (lInteger == other.lInteger) {
				return true;
			} else if (lInteger != null && other.lInteger != null) {
				return lInteger.equals(other.lInteger);
			}
		}
		return false;
	}

	/**
	 * Returns zero to satisfy the Object equals/hashCode contract. This class
	 * is mutable and not meant to be used as a hash key.
	 * 
	 * @return always zero
	 * @see Object#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	protected String getInternalString() throws IllegalValueException {
		if (lInteger != null) {
			return lInteger.toString();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	public Calendar getDate() throws IllegalValueException {
		setValueConsumed();

		if (lInteger != null) {
			// loosing timezone information...
			Calendar cal = Calendar.getInstance();
			cal.setTime(new Date(lInteger.longValue()));
			return cal;
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	public long getLong() throws IllegalValueException {
		setValueConsumed();
		if (lInteger != null) {
			return lInteger.longValue();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean getBoolean() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to boolean failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public double getDouble() throws IllegalValueException {
		setValueConsumed();

		if (lInteger != null) {
			return lInteger.doubleValue();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	public int getInt() throws IllegalValueException {
		setValueConsumed();
		return lInteger.intValue();
	}
}