package com.giantleap.common.nodebean.persistence.JDOM;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import com.giantleap.common.exception.GLException;
import com.giantleap.common.nodebean.node.base.storage.id.NodeId;
import com.giantleap.common.nodebean.node.base.storage.node.Node;
import com.giantleap.common.nodebean.node.base.storage.node.Property;
import com.giantleap.common.nodebean.node.base.storage.node.PropertyImp;
import com.giantleap.common.nodebean.node.base.storage.path.NameImp;
import com.giantleap.common.nodebean.node.base.storage.path.NameInterface;
import com.giantleap.common.nodebean.node.base.storage.path.PathInterface;
import com.giantleap.common.nodebean.node.base.storage.path.exception.IllegalNameException;
import com.giantleap.common.nodebean.node.base.storage.value.PropertyType;
import com.giantleap.common.nodebean.node.base.storage.value.Value;
import com.giantleap.common.nodebean.node.base.storage.value.ValueFactoryImp;
import com.giantleap.common.nodebean.node.root.RootNodeInterface;
import com.giantleap.common.nodebean.nodeCollection.NodeCollections;
import com.giantleap.common.nodebean.persistence.PersistenceManager;


public class JDOMPersistenceMgr extends PersistenceManager{

	public static final String JDOM_MANAGER = "JDOM";
	
	private static JDOMPersistenceMgr _JDOMPersistenceMgr = new JDOMPersistenceMgr();

	public static JDOMPersistenceMgr getInstance() {
		return _JDOMPersistenceMgr;
	}

	private JDOMPersistenceMgr() {

	}

	public Node loadStandalone(String uri) {		
		Element element = loadRootElement(uri);
		//System.out.println("element==null:"+(element!=null));
		if(element!=null){
			Node rootnode = loadNodeStandalone(element,null);
			if(rootnode instanceof RootNodeInterface){
				RootNodeInterface rnode=(RootNodeInterface)rootnode;
				rnode.setUri(uri);
				rnode.setPersistenceMgr(JDOM_MANAGER);
			}
			//System.out.println("uri:"+rootnode);
			return rootnode;
		}
		return null;
	}
	/*
	public RootNodeInterface loadExtend(String uri,MountableNode extendNode) {
		Element element = loadRootElement(uri);
		if(element!=null){
			//System.out.println("uri:"+uri);
			RootNodeInterface rootnode = loadMountableNode(element,extendNode);

			rootnode.setUri(uri);
			
			return rootnode;
		}
		return null;
	}*/
	
	

	
	public boolean save(RootNodeInterface rootNode) {
		Element rootElement = getElementByNode(rootNode,null);
		if(rootElement!=null){
			Document doc = new Document(rootElement);
			return storeXMLFile(rootNode.getUri(),doc);
		}
		return false;
	}

	public boolean save(Node rootNode,String uri) {
		Element rootElement = getElementByNode(rootNode,null);
		if(rootElement!=null){
			Document doc = new Document(rootElement);
			return storeXMLFile(uri,doc);
		}
		return false;
	}
	
	

	public static Element getElementByNode(Node node, Element parent) {
		if (node == null) throw new IllegalArgumentException("null");
		NameInterface name = node.getName();
		if (name == null) throw new IllegalArgumentException("null name");
		String nameString = name.getXMLString();
		Element ele = new Element(nameString);
		NodeId id = node.getId();
		List<Attribute> attList = getAttrbutes(node);
		if (id != null) {
			Attribute idAtt = new Attribute(Node.ID_PROPERTY_LITERAL, id.toString());
			boolean idexist = false;
			for (Attribute att : attList) {
				String attname = att.getName();
				if (attname.equals(Node.ID_PROPERTY_LITERAL)) idexist = true;
			}
			if (!idexist) attList.add(idAtt);
		}
		ele.setAttributes(attList);
		if(parent!=null)parent.addContent(ele);
		Collection<Node> childList = node.getAllChildNode();

		if (childList != null && childList.size() > 0) {
			for (Node childNode:childList) {

				//Node childNode = (Node) childList.get(i);
				if (childNode != null) {
					Element childEle = getElementByNode(childNode, ele);
				}
			}
		}
		return ele;
	}

	public static List<Attribute> getAttrbutes(Node node) {
		if (node == null) throw new IllegalArgumentException("null");
		List<Attribute> attlist = new ArrayList<Attribute>();
		Map<String, Property> properties = node.getPropertyMap();
		if (properties != null) {
			for (Iterator itId = properties.entrySet().iterator(); itId.hasNext();) {
				Map.Entry<String, Property> prop = (Map.Entry) itId.next();
				String proName = prop.getKey();
				Property property = prop.getValue();
				if (proName != null && property != null) {
					Value proValue = property.getValue();
					String valueStr = null;
					try {
						valueStr = proValue.getString();
					} catch (Exception e) {
						valueStr = "";
					}
					// System.out.println(valueStr);
					if (property.getValueType() == PropertyType.STRING) {
						Attribute attrib = new Attribute(proName, valueStr);
						attlist.add(attrib);
						// ele.setAttribute(proName, valueStr);
					} else {

						Attribute attrib = new Attribute(proName, valueStr);
						Attribute attribType = new Attribute(proName + Property.PROPERTY_TYPE_SUFFIX, String.valueOf(property.getValueType()));
						attlist.add(attrib);
						attlist.add(attribType);
					}
				}
			}
		}
		return attlist;
	}
	
	public static Node loadNodeStandalone(Element ele, Node parent){
		if (ele == null)return null;
		//System.out.println("aaaaaaaaaaaa");
		Node node = getNodeByElement(ele);
		if (node == null)return null;
		if(parent!=null){
			parent.addChildNode(node);
		}else{
			node.setPath(PathInterface.ROOT_PATH);
		}
		List childList = ele.getChildren();
		if (childList != null && childList.size() > 0) {
			for (int i = 0; i < childList.size(); i++) {
				Element childEle = (Element) childList.get(i);
				Node childNode = loadNode(childEle, node);
			}
		}
		return node;
	}
	

	/*
	public static MountableRootNode loadNodeExtend(Element ele, Node extendNode){
		if (ele == null)return null;
		//System.out.println("node == null:"+ele.toString());
		Node node = getNodeByElement(ele);
		if (node == null){
			//System.out.println("node == null");
			return null;
		}
		MountableRootNode rootnode = null;
		if(extendNode!=null && node instanceof MountableRootNode){
			rootnode = (MountableRootNode)node;
			rootnode.init(extendNode);		
			//rootnode.setExtendNode(extendNode);
			//rootnode.setPath(extendNode.getPath());
			
			//Node orTreeroot = extendNode.getRoot();
			//if(orTreeroot!=null && orTreeroot instanceof RootNodeInterface){
			//	RootNodeInterface rNode = (RootNodeInterface)orTreeroot;
			//	rootnode.setTreeId(rNode.getTreeId());
			//}
		}else{
			//System.out.println(node.getClass().toString());
			throw new IllegalArgumentException("extendNode==null"); 
		}
		List childList = ele.getChildren();
		if (childList != null && childList.size() > 0) {
			for (int i = 0; i < childList.size(); i++) {
				Element childEle = (Element) childList.get(i);
				Node childNode = loadNode(childEle, node);
			}
		}
		return rootnode;
	}
	*/
	
	public static Node loadNode(Element ele, Node parent){
		if (ele == null)return null;
		//System.out.println("aaaaaaaaaaaa");
		Node node = getNodeByElement(ele);
		if (node == null)return null;
		if(parent!=null){
			//System.out.println(node.getName().getXMLString());
			parent.addChildNode(node);
		}else{
			//if(isStandalone){
			node.setPath(PathInterface.ROOT_PATH);
			//}
		}
		List childList = ele.getChildren();
		if (childList != null && childList.size() > 0) {
			for (int i = 0; i < childList.size(); i++) {
				Element childEle = (Element) childList.get(i);
				Node childNode = loadNode(childEle, node);
			}
		}
		return node;
	}
	
	public static Node getNodeByElement(Element ele){
		if(ele==null) throw new IllegalArgumentException("null");
		String namestr = ele.getName();
		NameInterface name=null;
		try{
			name=NameImp.createName(namestr);
		}catch(IllegalNameException ex){
			ex.printStackTrace();
		}
		///System.out.println(name.getXMLString());
		Map<String,Property> properties = getPropertyMap(ele);

		//NodeFactory fac=NodeFactoryImp.getInstance();
		NodeCollections collections = NodeCollections.getInstance();
		
		Node node=collections.createNode(name,properties);
		//System.out.println(node);
		return node;
	}
	
	public static Map<String,Property> getPropertyMap(Element ele){
		Map<String,Property> propertyMap =new HashMap<String,Property>();
		if(ele==null)  throw new IllegalArgumentException("null");
		List attribList=ele.getAttributes();
		Map typeMap=new HashMap();
		Map proMap =new HashMap();
		for(int i=0;i<attribList.size();i++){
			Attribute attrib = (Attribute)attribList.get(i);
			String attrbiName=attrib.getName();
			String value     =attrib.getValue();
			if(attrbiName.contains(Property.PROPERTY_TYPE_SUFFIX)){
				int tmp=attrbiName.indexOf(Property.PROPERTY_TYPE_SUFFIX);
				String typeName = attrbiName.substring(0,tmp);
				typeMap.put(typeName,value);
			}else{
				proMap.put(attrbiName, value);
			}
		}
		
		if(proMap.size()<1) return propertyMap;
	    for (Iterator itId=proMap.entrySet().iterator(); itId.hasNext(); ) {
	    	Property property = null;
	    	Map.Entry<String,String> prop = (Map.Entry)itId.next();
	        String proName = prop.getKey();	        
	        String proValueString = prop.getValue();	        
	        try{
		        NameInterface name=NameImp.createName(proName);
		        if(typeMap.containsKey(proName)){		        	
		        	String proTypeString=(String)typeMap.get(proName);
		        	Value proValue=ValueFactoryImp.createValue(proValueString, proTypeString);		        	
		        	property=PropertyImp.create(name,proValue);
		        }else{
		        	Value proValue=ValueFactoryImp.createValue(proValueString, PropertyType.STRING);
		        	property=PropertyImp.create(name,proValue);
		        }
	        }catch(GLException e){
	        	e.printStackTrace();
	        	continue;
	        }
	        propertyMap.put(proName,property);
	    }
	    return propertyMap;
	}
	
	public static boolean storeXMLFile(String fileName,Document doc){
		try {
			File file = new File(fileName);
			File parent  = file.getParentFile();
			if(!parent.exists()) parent.mkdirs();
			
			
			Format format = Format.getCompactFormat();
	        format.setEncoding("UTF-8");           
	        format.setIndent("    ");               
	        XMLOutputter XMLOut = new XMLOutputter(format);
	        FileOutputStream out=new FileOutputStream(fileName);
	        XMLOut.output(doc, out); 
	        out.close();
	        return true;
		}catch(Exception e){
			e.printStackTrace();
			return false;
		}		
	}
	
	public static Element loadRootElement(String fileName){
		if(fileName==null) throw new IllegalArgumentException("null");
		fileName = fileName.trim();
		//System.out.println("!file.exists():" + fileName);
		File file=new File(fileName);
		//System.out.println("!file.exists():"+(!file.exists()));
		if(!file.exists()){		
			return null;
		}
		SAXBuilder builder = new SAXBuilder();
		try{
			Document doc = builder.build(file);	
			return doc.getRootElement();	
		}catch(Exception e){
			e.printStackTrace();
			return null;
		}		
	}
}
