package com.giantleap.common.util.dataformat;

import java.util.Calendar;
import java.util.Date;

public class CalendarFormat {

	public static Calendar NULL_CALENDAR=Calendar.getInstance();
	static{
		NULL_CALENDAR.setTimeInMillis(0L);
	}
	
	public static String getCalendarDateString(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		int year  = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		sb3.append(String.valueOf(year));
		sb3.append("-");
		if(month<10) sb3.append("0");
		sb3.append(String.valueOf(month));
		sb3.append("-");
		if(day<10) sb3.append("0");
		sb3.append(String.valueOf(day));
		return sb3.toString();
	}
	
	public static Calendar parseCalendarString(String str){
		if(str==null) throw new IllegalArgumentException("null");
		//System.out.println(str.length());
		//if(str.length()<11)return null;
		str = str.trim();
		if(!str.contains("-"))return null;
		Calendar cal=Calendar.getInstance();
		String[] strs = str.split("-");
		//System.out.println(strs.length);
		if(strs.length!=3)return null;
		
		String yearstr=strs[0];
		String monthstr=strs[1];
		String daystr=strs[2];
		try{
			int year=Integer.parseInt(yearstr);
			int month=Integer.parseInt(monthstr);
			int day=Integer.parseInt(daystr);
			cal.set(Calendar.YEAR,year);
			cal.set(Calendar.MONTH,month-1);
			cal.set(Calendar.DAY_OF_MONTH,day);
		}catch(NumberFormatException e){
			return null;
		}
		return cal;
	}
	
	public static String getCalendarDateString(Date date){
		Calendar cal=Calendar.getInstance();
		cal.setTime(date);
		return getCalendarDateString(cal);
	}
	
	public static String getCalendarTimeString(Date date){
		Calendar cal=Calendar.getInstance();
		cal.setTime(date);
		return getCalendarTimeString(cal);
	}
	
	public static String getCalendarTimeString(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		int year  = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		int hour  = cal.get(Calendar.HOUR_OF_DAY);
		int minute= cal.get(Calendar.MINUTE);
		sb3.append(String.valueOf(year));
		sb3.append("-");
		if(month<10) sb3.append("0");
		sb3.append(String.valueOf(month));
		sb3.append("-");
		if(day<10) sb3.append("0");
		sb3.append(String.valueOf(day));
		sb3.append(" ");
		if(hour<10) sb3.append("0");
		sb3.append(String.valueOf(hour));
		sb3.append(":");
		if(minute<10) sb3.append("0");
		sb3.append(String.valueOf(minute));
		return sb3.toString();
	}
	public static String getCalendarTimeString6(Date date){
		Calendar cal=Calendar.getInstance();
		cal.setTime(date);
		return getCalendarTimeString6(cal);
	}
	public static String getCalendarTimeString6(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		int year  = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		int hour  = cal.get(Calendar.HOUR_OF_DAY);
		int minute= cal.get(Calendar.MINUTE);
		int second= cal.get(Calendar.SECOND);
		sb3.append(String.valueOf(year));
		sb3.append("-");
		if(month<10) sb3.append("0");
		sb3.append(String.valueOf(month));
		sb3.append("-");
		if(day<10) sb3.append("0");
		sb3.append(String.valueOf(day));
		sb3.append(" ");
		if(hour<10) sb3.append("0");
		sb3.append(String.valueOf(hour));
		sb3.append(":");
		if(minute<10) sb3.append("0");
		sb3.append(String.valueOf(minute));
		sb3.append(":");
		if(second<10) sb3.append("0");
		sb3.append(String.valueOf(second));
		return sb3.toString();
	}
	
	public static String getCalendarPath(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		int year  = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		sb3.append(year);
		sb3.append("/");
		sb3.append(month);
		sb3.append("/");
		sb3.append(day);
		sb3.append("/");
		return sb3.toString();
	}
	
	public static String getCalendarFilename(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		int year  = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		sb3.append(year);
		sb3.append("_");
		sb3.append(month);
		sb3.append("_");
		sb3.append(day);
		//sb3.append("_");
		return sb3.toString();
	}
	
	public static String getCalendarTimeString3(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		//int year  = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		int hour  = cal.get(Calendar.HOUR_OF_DAY);
		int minute= cal.get(Calendar.MINUTE);
		//sb3.append(String.valueOf(year));
		//sb3.append("-");
		//if(month<10) sb3.append("0");
		sb3.append(String.valueOf(month));
		sb3.append("月");
		//if(day<10) sb3.append("0");
		sb3.append(String.valueOf(day));
		sb3.append("日");
		if(month<10) sb3.append(" ");
		if(day<10) sb3.append(" ");
		sb3.append(" ");
		if(hour<10) sb3.append("0");
		sb3.append(String.valueOf(hour));
		sb3.append(":");
		if(minute<10) sb3.append("0");
		sb3.append(String.valueOf(minute));
		return sb3.toString();
	}
	
	public static String getCalendarTimeString2(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		int hour  = cal.get(Calendar.HOUR_OF_DAY);
		int minute= cal.get(Calendar.MINUTE);
		if(month<10) sb3.append("0");
		sb3.append(String.valueOf(month));
		sb3.append("月");
		if(day<10) sb3.append("0");
		sb3.append(String.valueOf(day));
		sb3.append("日 ");
		if(hour<10) sb3.append("0");
		sb3.append(String.valueOf(hour));
		sb3.append(":");
		if(minute<10) sb3.append("0");
		sb3.append(String.valueOf(minute));
		return sb3.toString();
	}
	public static String getCalendarTimeString4(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		if(month<10) sb3.append("0");
		sb3.append(String.valueOf(month));
		sb3.append("月");
		if(day<10) sb3.append("0");
		sb3.append(String.valueOf(day));
		sb3.append("日 ");
		return sb3.toString();
	}
	
	public static String getCalendarTimeString5(Calendar cal){
		if(cal==null) throw new IllegalArgumentException("null");
		StringBuffer sb3=new StringBuffer();
		int year   = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		sb3.append(year);
		sb3.append("-");
		if(month<10) sb3.append("0");
		sb3.append(String.valueOf(month));
		sb3.append("-");
		if(day<10) sb3.append("0");
		sb3.append(String.valueOf(day));
		
		return sb3.toString();
	}
	public static String getCalendarTimeString(long time){
		Date date=new Date(time);
		//Calendar cal=Calendar.getInstance();
		//cal.setTime(date);
		return getCalendarTimeString(date);
	}
	
	
	public static Calendar getCalendar(String str){
		if(str==null) throw new IllegalArgumentException("null");
		str=str.trim();
		Calendar cal=Calendar.getInstance();
		String yearstr=str.substring(0,4);
		String monthstr=str.substring(5,7);
		String daystr=str.substring(8,10);
		String hourstr=str.substring(11,13);
		String minstr=str.substring(14,16);
		String secstr=str.substring(17,19);

		try{
			int year=Integer.parseInt(yearstr);
			int month=Integer.parseInt(monthstr);
			int day=Integer.parseInt(daystr);
			int hour=Integer.parseInt(hourstr);
			int min=Integer.parseInt(minstr);
			int sec=Integer.parseInt(secstr);
			cal.set(Calendar.YEAR,year);
			cal.set(Calendar.MONTH,month-1);
			cal.set(Calendar.DAY_OF_MONTH,day);
			cal.set(Calendar.HOUR_OF_DAY,hour);
			cal.set(Calendar.MINUTE,min);
			cal.set(Calendar.SECOND,sec);
		}catch(NumberFormatException e){
			return NULL_CALENDAR;
		}
		//System.out.println(secstr);
		
		return cal;
	}
	
	public static Calendar getCalendar2(String str){
		if(str==null) throw new IllegalArgumentException("null");
		Calendar cal=Calendar.getInstance();
		String strYear=str.substring(0,str.indexOf('-'));
		String strMonth=str.substring(str.indexOf('-')+1,str.lastIndexOf('-'));
		String strDay=str.substring(str.lastIndexOf('-')+1);
		//System.out.println(strYear);
		//System.out.println(strMonth);
		//System.out.println(strDay);
		try{
			int year=Integer.parseInt(strYear);
			int month=Integer.parseInt(strMonth);
			int day=Integer.parseInt(strDay);
			cal.set(Calendar.YEAR,year);
			cal.set(Calendar.MONTH,month-1);
			cal.set(Calendar.DAY_OF_MONTH,day);
		}catch(NumberFormatException e){
			return NULL_CALENDAR;
		}
		//System.out.println(cal);
		return cal;
	}
	
	public static Calendar getCalendar3(String str){
		if(str==null) throw new IllegalArgumentException("null");
		str=str.trim();
		Calendar cal=Calendar.getInstance();
		
		String datestr=str.substring(0, str.indexOf(" "));
		String timestr=str.substring(str.indexOf(" ")+1);
		//System.out.println(datestr);
		String strYear=datestr.substring(0,datestr.indexOf('-'));
		String strMonth=datestr.substring(datestr.indexOf('-')+1,datestr.lastIndexOf('-'));
		String strDay=datestr.substring(datestr.lastIndexOf('-')+1);
		
		String strHour=timestr.substring(0,timestr.indexOf(':'));
		String strMin=timestr.substring(timestr.indexOf(':')+1);
		//System.out.println(strYear);
		//System.out.println(strMonth);
		//System.out.println(strDay);
		try{
			int year=Integer.parseInt(strYear);
			int month=Integer.parseInt(strMonth);
			int day=Integer.parseInt(strDay);
			
			int hour=Integer.parseInt(strHour);
			int min=Integer.parseInt(strMin);
			
			cal.set(Calendar.YEAR,year);
			cal.set(Calendar.MONTH,month-1);
			cal.set(Calendar.DAY_OF_MONTH,day);
			cal.set(Calendar.HOUR_OF_DAY,hour);
			cal.set(Calendar.MINUTE,min);
		}catch(NumberFormatException e){
			return NULL_CALENDAR;
		}
		//System.out.println(cal);
		return cal;
	}
	
	public static Calendar getCalendar4(String str){
		if(str==null) throw new IllegalArgumentException("null");
		str=str.trim();
		Calendar cal=Calendar.getInstance();
		
		String datestr=str.substring(0, str.indexOf(" "));
		String timestr=str.substring(str.indexOf(" ")+1);
		//System.out.println(datestr);
		String strYear=datestr.substring(0,datestr.indexOf('-'));
		String strMonth=datestr.substring(datestr.indexOf('-')+1,datestr.lastIndexOf('-'));
		String strDay=datestr.substring(datestr.lastIndexOf('-')+1);
		
		String strHour=timestr.substring(0,timestr.indexOf(':'));
		String strMin=timestr.substring(timestr.indexOf(':')+1,timestr.lastIndexOf(':'));
		String strSec=timestr.substring(timestr.lastIndexOf(':')+1);
		//System.out.println(strYear);
		//System.out.println(strMonth);
		//System.out.println(strDay);
		try{
			int year=Integer.parseInt(strYear);
			int month=Integer.parseInt(strMonth);
			int day=Integer.parseInt(strDay);
			
			int hour=Integer.parseInt(strHour);
			int min=Integer.parseInt(strMin);
			int sec=Integer.parseInt(strSec);
			cal.set(Calendar.YEAR,year);
			cal.set(Calendar.MONTH,month-1);
			cal.set(Calendar.DAY_OF_MONTH,day);
			cal.set(Calendar.HOUR_OF_DAY,hour);
			cal.set(Calendar.MINUTE,min);
			cal.set(Calendar.SECOND,sec);
		}catch(NumberFormatException e){
			return NULL_CALENDAR;
		}
		//System.out.println(cal);
		return cal;
	}
	
	public static Calendar getCalendar5(String posttimeString){
		//if(str==null) throw new IllegalArgumentException("null");
		if(posttimeString==null || posttimeString.trim().equals("")){
			return null;
		}
		Calendar cal=Calendar.getInstance();
		String dateString = posttimeString.substring(0,posttimeString.indexOf(" "));
		String timeString = posttimeString.substring(posttimeString.indexOf(" ")+1);
		
		String yearString  = dateString.substring(0,dateString.indexOf("年"));
		String monthString = dateString.substring(dateString.indexOf("年")+1,dateString.indexOf("月"));
		String dayString   = dateString.substring(dateString.indexOf("月")+1,dateString.indexOf("日"));
		
		String hourString  = timeString.substring(0,timeString.indexOf(":"));
		String minString   = timeString.substring(timeString.indexOf(":")+1);
		try{
			int year=Integer.parseInt(yearString);
			int month=Integer.parseInt(monthString);
			int day=Integer.parseInt(dayString);
			
			int hour=Integer.parseInt(hourString);
			int min=Integer.parseInt(minString);
			cal.set(Calendar.YEAR,year);
			cal.set(Calendar.MONTH,month-1);
			cal.set(Calendar.DAY_OF_MONTH,day);
			cal.set(Calendar.HOUR_OF_DAY,hour);
			cal.set(Calendar.MINUTE,min);
		}catch(NumberFormatException e){
			return NULL_CALENDAR;
		}
		//System.out.println(cal);
		return cal;
	}
	
	public static Calendar getTodayStart(){
		Calendar cal=Calendar.getInstance();
		cal.setTime(new Date());
		int year  = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH)+1;
		int day   = cal.get(Calendar.DAY_OF_MONTH);
		int hour=0;
		int min=0;
		Calendar cal2=Calendar.getInstance();
		cal2.set(Calendar.YEAR,year);
		cal2.set(Calendar.MONTH,month-1);
		cal2.set(Calendar.DAY_OF_MONTH,day);
		cal2.set(Calendar.HOUR_OF_DAY,hour);
		cal2.set(Calendar.MINUTE,min);
		return cal2;
	}
	
	public static Calendar getTW4Hrs(){
		Calendar cal=Calendar.getInstance();
		cal.setTime(new Date());
		cal.roll(Calendar.DATE, false);
		return cal;
	}
	
	public static Calendar getSevenDays(){
		Calendar cal=Calendar.getInstance();
		cal.setTime(new Date());
		cal.roll(Calendar.DATE, -7);
		return cal;
	}
	
	public static void main(String[] args){
		CalendarFormat.getCalendar("2009年01月10日20:03:43 星期六");
	}
}
